"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema.exceptions import ValidationError
from cfnlint.jsonschema.validators import CfnTemplateValidator


def fail(validator, errors, instance, schema):
    raise ValueError(f"Bad {errors!r}")


@pytest.fixture
def validator():
    validators = {
        "foo": fail,
        "bar": fail,
    }
    validator = CfnTemplateValidator({}).extend(
        validators=validators,
    )({"foo": True, "bar": False})
    return validator


def test_validator_raises_exception(validator):

    errs = list(validator.iter_errors(""))

    assert errs == [
        ValidationError(
            "Exception 'Bad True' raised while validating 'foo'",
            validator="foo",
            schema_path=deque(["foo"]),
            path=deque([]),
        ),
        ValidationError(
            "Exception 'Bad False' raised while validating 'bar'",
            validator="bar",
            schema_path=deque(["bar"]),
            path=deque([]),
        ),
    ]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import cfnlint.data.schemas.extensions.aws_elasticloadbalancing_loadbalancer
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class Certificate(CfnLintJsonSchema):
    id = "E3679"
    shortdesc = (
        "Validate ELB protocols that require certificates have a certificate specified"
    )
    description = "When using HTTPS or SSL you must provide a certificate"
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancing-loadbalancer-listeners.html#cfn-elasticloadbalancing-loadbalancer-listeners-sslcertificateid"
    tags = ["resources"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::ElasticLoadBalancing::LoadBalancer/Properties/Listeners/*",
            ],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_elasticloadbalancing_loadbalancer,
                filename="certificate.json",
            ),
            all_matches=True,
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules._rule import CloudFormationLintRule, Match, RuleMatch
from cfnlint.rules._rules import Rules, RulesCollection
from cfnlint.rules.jsonschema import (
    CfnLintJsonSchema,
    CfnLintJsonSchemaRegional,
    CfnLintKeyword,
    SchemaDetails,
)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

from cfnlint.jsonschema import ValidationError, ValidationResult, Validator
from cfnlint.rules.jsonschema.CfnLintKeyword import CfnLintKeyword


class RuleScheduleExpression(CfnLintKeyword):
    """Validate AWS Events Schedule expression format"""

    id = "E3027"
    shortdesc = "Validate AWS Event ScheduleExpression format"
    description = "Validate the formation of the AWS::Event ScheduleExpression"
    source_url = "https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/ScheduledEvents.html"
    tags = ["resources", "events"]

    def initialize(self, cfn):
        """Initialize the rule"""
        self.__init__(["Resources/AWS::Events::Rule/Properties/ScheduleExpression"])

    def validate(
        self, validator: Validator, keywords: Any, instance: Any, schema: dict[str, Any]
    ) -> ValidationResult:
        # Value is either "cron()" or "rate()"
        if not validator.is_type(instance, "string"):
            return

        if instance.startswith("rate(") and instance.endswith(")"):
            yield from self._check_rate(instance)
        elif instance.startswith("cron(") and instance.endswith(")"):
            yield from self._check_cron(instance)
        else:
            yield ValidationError(f"{instance!r} has to be either 'cron()' or 'rate()'")

    def _check_rate(self, value):
        """Check Rate configuration"""
        # Extract the expression from rate(XXX)
        rate_expression = value[value.find("(") + 1 : value.find(")")]

        if not rate_expression:
            yield ValidationError(
                message=f"{rate_expression!r} is not of type 'string'",
            )
            return

        # Rate format: rate(Value Unit)
        items = rate_expression.split(" ")

        if len(items) != 2:
            yield ValidationError(
                f"{rate_expression!r} has to be of format rate(Value Unit)"
            )
            return

        # Check the Value
        if not items[0].isdigit():
            extra_args = {
                "actual_type": type(items[0]).__name__,
                "expected_type": int.__name__,
            }
            yield ValidationError(
                message=f"{items[0]!r} is not of type 'integer'",
                extra_args=extra_args,
            )
            return

        if float(items[0]) <= 0:
            yield ValidationError(f"{items[0]!r} is less than the minimum of {0!r}")

        if float(items[0]) <= 1:
            valid_periods = ["minute", "hour", "day"]
        elif float(items[0]) > 1:
            valid_periods = ["minutes", "hours", "days"]
        # Check the Unit
        if items[1] not in valid_periods:
            yield ValidationError(f"{items[1]!r} is not one of {valid_periods!r}")

    def _check_cron(self, value):
        """Check Cron configuration"""
        # Extract the expression from cron(XXX)
        cron_expression = value[value.find("(") + 1 : value.find(")")]

        if not cron_expression:
            yield ValidationError(
                message=f"{cron_expression!r} is not of type 'string'",
            )
            return
        else:
            # Rate format: cron(Minutes Hours Day-of-month Month Day-of-week Year)
            items = cron_expression.split(" ")

            if len(items) != 6:
                yield ValidationError(
                    message=(
                        f"{items[0]!r} is not of length {6!r}. "
                        "(Minutes Hours Day-of-month Month Day-of-week Year)"
                    ),
                )
                return

            _, _, day_of_month, _, day_of_week, _ = cron_expression.split(" ")
            if day_of_month != "?" and day_of_week != "?":
                yield ValidationError(
                    message=(
                        f"{cron_expression[0]!r} specifies both "
                        "Day-of-month and Day-of-week. "
                        "(Minutes Hours Day-of-month Month Day-of-week Year)"
                    ),
                )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.jsonschema import ValidationError, _utils
from cfnlint.rules import CloudFormationLintRule


class UniqueItems(CloudFormationLintRule):
    """Check if duplicates exist in a List"""

    id = "E3037"
    shortdesc = "Check if a list has duplicate values"
    description = (
        "Certain lists don't support duplicate items. "
        "Check when duplicates are provided but not supported."
    )
    source_url = "https://github.com/aws-cloudformation/cfn-lint/blob/main/docs/cfn-schema-specification.md#uniqueitems"
    tags = ["resources", "property", "list"]
    child_rules = {
        "I3037": None,
    }

    # pylint: disable=unused-argument
    def uniqueItems(self, validator, uI, instance, schema):
        if not validator.is_type(instance, "array"):
            return
        if not _utils.uniq(instance):
            if uI:
                yield ValidationError(f"{instance!r} has non-unique elements")
            elif self.child_rules["I3037"]:
                yield ValidationError(
                    f"{instance!r} has non-unique elements",
                    rule=self.child_rules["I3037"],
                )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.formatters.base import BaseFormatter
from cfnlint.formatters.formatter import Formatter
from cfnlint.formatters.json import JsonFormatter
from cfnlint.formatters.junit import JUnitFormatter
from cfnlint.formatters.parseable import ParseableFormatter
from cfnlint.formatters.pretty import PrettyFormatter
from cfnlint.formatters.quit import QuietFormatter
from cfnlint.formatters.sarif import SARIFFormatter



from __future__ import annotations

# pylint: disable=too-many-lines
types: list[str] = [
    "AWS::ACMPCA::Certificate",
    "AWS::ACMPCA::CertificateAuthority",
    "AWS::ACMPCA::CertificateAuthorityActivation",
    "AWS::ACMPCA::Permission",
    "AWS::APS::RuleGroupsNamespace",
    "AWS::APS::Scraper",
    "AWS::APS::Workspace",
    "AWS::ARCZonalShift::ZonalAutoshiftConfiguration",
    "AWS::AccessAnalyzer::Analyzer",
    "AWS::AmazonMQ::Broker",
    "AWS::AmazonMQ::Configuration",
    "AWS::AmazonMQ::ConfigurationAssociation",
    "AWS::Amplify::App",
    "AWS::Amplify::Branch",
    "AWS::Amplify::Domain",
    "AWS::AmplifyUIBuilder::Component",
    "AWS::AmplifyUIBuilder::Form",
    "AWS::AmplifyUIBuilder::Theme",
    "AWS::ApiGateway::Account",
    "AWS::ApiGateway::ApiKey",
    "AWS::ApiGateway::Authorizer",
    "AWS::ApiGateway::BasePathMapping",
    "AWS::ApiGateway::ClientCertificate",
    "AWS::ApiGateway::Deployment",
    "AWS::ApiGateway::DocumentationPart",
    "AWS::ApiGateway::DocumentationVersion",
    "AWS::ApiGateway::DomainName",
    "AWS::ApiGateway::GatewayResponse",
    "AWS::ApiGateway::Method",
    "AWS::ApiGateway::Model",
    "AWS::ApiGateway::RequestValidator",
    "AWS::ApiGateway::Resource",
    "AWS::ApiGateway::RestApi",
    "AWS::ApiGateway::Stage",
    "AWS::ApiGateway::UsagePlan",
    "AWS::ApiGateway::UsagePlanKey",
    "AWS::ApiGateway::VpcLink",
    "AWS::ApiGatewayV2::Api",
    "AWS::ApiGatewayV2::ApiGatewayManagedOverrides",
    "AWS::ApiGatewayV2::ApiMapping",
    "AWS::ApiGatewayV2::Authorizer",
    "AWS::ApiGatewayV2::Deployment",
    "AWS::ApiGatewayV2::DomainName",
    "AWS::ApiGatewayV2::Integration",
    "AWS::ApiGatewayV2::IntegrationResponse",
    "AWS::ApiGatewayV2::Model",
    "AWS::ApiGatewayV2::Route",
    "AWS::ApiGatewayV2::RouteResponse",
    "AWS::ApiGatewayV2::Stage",
    "AWS::ApiGatewayV2::VpcLink",
    "AWS::AppConfig::Application",
    "AWS::AppConfig::ConfigurationProfile",
    "AWS::AppConfig::Deployment",
    "AWS::AppConfig::DeploymentStrategy",
    "AWS::AppConfig::Environment",
    "AWS::AppConfig::Extension",
    "AWS::AppConfig::ExtensionAssociation",
    "AWS::AppConfig::HostedConfigurationVersion",
    "AWS::AppFlow::Connector",
    "AWS::AppFlow::ConnectorProfile",
    "AWS::AppFlow::Flow",
    "AWS::AppIntegrations::Application",
    "AWS::AppIntegrations::DataIntegration",
    "AWS::AppIntegrations::EventIntegration",
    "AWS::AppMesh::GatewayRoute",
    "AWS::AppMesh::Mesh",
    "AWS::AppMesh::Route",
    "AWS::AppMesh::VirtualGateway",
    "AWS::AppMesh::VirtualNode",
    "AWS::AppMesh::VirtualRouter",
    "AWS::AppMesh::VirtualService",
    "AWS::AppRunner::AutoScalingConfiguration",
    "AWS::AppRunner::ObservabilityConfiguration",
    "AWS::AppRunner::Service",
    "AWS::AppRunner::VpcConnector",
    "AWS::AppRunner::VpcIngressConnection",
    "AWS::AppStream::AppBlock",
    "AWS::AppStream::AppBlockBuilder",
    "AWS::AppStream::Application",
    "AWS::AppStream::ApplicationEntitlementAssociation",
    "AWS::AppStream::ApplicationFleetAssociation",
    "AWS::AppStream::DirectoryConfig",
    "AWS::AppStream::Entitlement",
    "AWS::AppStream::Fleet",
    "AWS::AppStream::ImageBuilder",
    "AWS::AppStream::Stack",
    "AWS::AppStream::StackFleetAssociation",
    "AWS::AppStream::StackUserAssociation",
    "AWS::AppStream::User",
    "AWS::AppSync::ApiCache",
    "AWS::AppSync::ApiKey",
    "AWS::AppSync::DataSource",
    "AWS::AppSync::DomainName",
    "AWS::AppSync::DomainNameApiAssociation",
    "AWS::AppSync::FunctionConfiguration",
    "AWS::AppSync::GraphQLApi",
    "AWS::AppSync::GraphQLSchema",
    "AWS::AppSync::Resolver",
    "AWS::AppSync::SourceApiAssociation",
    "AWS::AppTest::TestCase",
    "AWS::ApplicationAutoScaling::ScalableTarget",
    "AWS::ApplicationAutoScaling::ScalingPolicy",
    "AWS::ApplicationInsights::Application",
    "AWS::ApplicationSignals::ServiceLevelObjective",
    "AWS::Athena::CapacityReservation",
    "AWS::Athena::DataCatalog",
    "AWS::Athena::NamedQuery",
    "AWS::Athena::PreparedStatement",
    "AWS::Athena::WorkGroup",
    "AWS::AuditManager::Assessment",
    "AWS::AutoScaling::AutoScalingGroup",
    "AWS::AutoScaling::LaunchConfiguration",
    "AWS::AutoScaling::LifecycleHook",
    "AWS::AutoScaling::ScalingPolicy",
    "AWS::AutoScaling::ScheduledAction",
    "AWS::AutoScaling::WarmPool",
    "AWS::AutoScalingPlans::ScalingPlan",
    "AWS::B2BI::Capability",
    "AWS::B2BI::Partnership",
    "AWS::B2BI::Profile",
    "AWS::B2BI::Transformer",
    "AWS::BCMDataExports::Export",
    "AWS::Backup::BackupPlan",
    "AWS::Backup::BackupSelection",
    "AWS::Backup::BackupVault",
    "AWS::Backup::Framework",
    "AWS::Backup::ReportPlan",
    "AWS::Backup::RestoreTestingPlan",
    "AWS::Backup::RestoreTestingSelection",
    "AWS::BackupGateway::Hypervisor",
    "AWS::Batch::ComputeEnvironment",
    "AWS::Batch::JobDefinition",
    "AWS::Batch::JobQueue",
    "AWS::Batch::SchedulingPolicy",
    "AWS::Bedrock::Agent",
    "AWS::Bedrock::AgentAlias",
    "AWS::Bedrock::DataSource",
    "AWS::Bedrock::Guardrail",
    "AWS::Bedrock::GuardrailVersion",
    "AWS::Bedrock::KnowledgeBase",
    "AWS::BillingConductor::BillingGroup",
    "AWS::BillingConductor::CustomLineItem",
    "AWS::BillingConductor::PricingPlan",
    "AWS::BillingConductor::PricingRule",
    "AWS::Budgets::Budget",
    "AWS::Budgets::BudgetsAction",
    "AWS::CDK::Metadata",
    "AWS::CE::AnomalyMonitor",
    "AWS::CE::AnomalySubscription",
    "AWS::CE::CostCategory",
    "AWS::CUR::ReportDefinition",
    "AWS::Cassandra::Keyspace",
    "AWS::Cassandra::Table",
    "AWS::CertificateManager::Account",
    "AWS::CertificateManager::Certificate",
    "AWS::Chatbot::MicrosoftTeamsChannelConfiguration",
    "AWS::Chatbot::SlackChannelConfiguration",
    "AWS::CleanRooms::AnalysisTemplate",
    "AWS::CleanRooms::Collaboration",
    "AWS::CleanRooms::ConfiguredTable",
    "AWS::CleanRooms::ConfiguredTableAssociation",
    "AWS::CleanRooms::Membership",
    "AWS::CleanRooms::PrivacyBudgetTemplate",
    "AWS::CleanRoomsML::TrainingDataset",
    "AWS::Cloud9::EnvironmentEC2",
    "AWS::CloudFormation::CustomResource",
    "AWS::CloudFormation::HookDefaultVersion",
    "AWS::CloudFormation::HookTypeConfig",
    "AWS::CloudFormation::HookVersion",
    "AWS::CloudFormation::Macro",
    "AWS::CloudFormation::ModuleDefaultVersion",
    "AWS::CloudFormation::ModuleVersion",
    "AWS::CloudFormation::PublicTypeVersion",
    "AWS::CloudFormation::Publisher",
    "AWS::CloudFormation::ResourceDefaultVersion",
    "AWS::CloudFormation::ResourceVersion",
    "AWS::CloudFormation::Stack",
    "AWS::CloudFormation::StackSet",
    "AWS::CloudFormation::TypeActivation",
    "AWS::CloudFormation::WaitCondition",
    "AWS::CloudFormation::WaitConditionHandle",
    "AWS::CloudFront::CachePolicy",
    "AWS::CloudFront::CloudFrontOriginAccessIdentity",
    "AWS::CloudFront::ContinuousDeploymentPolicy",
    "AWS::CloudFront::Distribution",
    "AWS::CloudFront::Function",
    "AWS::CloudFront::KeyGroup",
    "AWS::CloudFront::KeyValueStore",
    "AWS::CloudFront::MonitoringSubscription",
    "AWS::CloudFront::OriginAccessControl",
    "AWS::CloudFront::OriginRequestPolicy",
    "AWS::CloudFront::PublicKey",
    "AWS::CloudFront::RealtimeLogConfig",
    "AWS::CloudFront::ResponseHeadersPolicy",
    "AWS::CloudFront::StreamingDistribution",
    "AWS::CloudTrail::Channel",
    "AWS::CloudTrail::EventDataStore",
    "AWS::CloudTrail::ResourcePolicy",
    "AWS::CloudTrail::Trail",
    "AWS::CloudWatch::Alarm",
    "AWS::CloudWatch::AnomalyDetector",
    "AWS::CloudWatch::CompositeAlarm",
    "AWS::CloudWatch::Dashboard",
    "AWS::CloudWatch::InsightRule",
    "AWS::CloudWatch::MetricStream",
    "AWS::CodeArtifact::Domain",
    "AWS::CodeArtifact::PackageGroup",
    "AWS::CodeArtifact::Repository",
    "AWS::CodeBuild::Fleet",
    "AWS::CodeBuild::Project",
    "AWS::CodeBuild::ReportGroup",
    "AWS::CodeBuild::SourceCredential",
    "AWS::CodeCommit::Repository",
    "AWS::CodeConnections::Connection",
    "AWS::CodeDeploy::Application",
    "AWS::CodeDeploy::DeploymentConfig",
    "AWS::CodeDeploy::DeploymentGroup",
    "AWS::CodeGuruProfiler::ProfilingGroup",
    "AWS::CodeGuruReviewer::RepositoryAssociation",
    "AWS::CodePipeline::CustomActionType",
    "AWS::CodePipeline::Pipeline",
    "AWS::CodePipeline::Webhook",
    "AWS::CodeStar::GitHubRepository",
    "AWS::CodeStarConnections::Connection",
    "AWS::CodeStarConnections::RepositoryLink",
    "AWS::CodeStarConnections::SyncConfiguration",
    "AWS::CodeStarNotifications::NotificationRule",
    "AWS::Cognito::IdentityPool",
    "AWS::Cognito::IdentityPoolPrincipalTag",
    "AWS::Cognito::IdentityPoolRoleAttachment",
    "AWS::Cognito::LogDeliveryConfiguration",
    "AWS::Cognito::UserPool",
    "AWS::Cognito::UserPoolClient",
    "AWS::Cognito::UserPoolDomain",
    "AWS::Cognito::UserPoolGroup",
    "AWS::Cognito::UserPoolIdentityProvider",
    "AWS::Cognito::UserPoolResourceServer",
    "AWS::Cognito::UserPoolRiskConfigurationAttachment",
    "AWS::Cognito::UserPoolUICustomizationAttachment",
    "AWS::Cognito::UserPoolUser",
    "AWS::Cognito::UserPoolUserToGroupAttachment",
    "AWS::Comprehend::DocumentClassifier",
    "AWS::Comprehend::Flywheel",
    "AWS::Config::AggregationAuthorization",
    "AWS::Config::ConfigRule",
    "AWS::Config::ConfigurationAggregator",
    "AWS::Config::ConfigurationRecorder",
    "AWS::Config::ConformancePack",
    "AWS::Config::DeliveryChannel",
    "AWS::Config::OrganizationConfigRule",
    "AWS::Config::OrganizationConformancePack",
    "AWS::Config::RemediationConfiguration",
    "AWS::Config::StoredQuery",
    "AWS::Connect::ApprovedOrigin",
    "AWS::Connect::ContactFlow",
    "AWS::Connect::ContactFlowModule",
    "AWS::Connect::EvaluationForm",
    "AWS::Connect::HoursOfOperation",
    "AWS::Connect::Instance",
    "AWS::Connect::InstanceStorageConfig",
    "AWS::Connect::IntegrationAssociation",
    "AWS::Connect::PhoneNumber",
    "AWS::Connect::PredefinedAttribute",
    "AWS::Connect::Prompt",
    "AWS::Connect::Queue",
    "AWS::Connect::QuickConnect",
    "AWS::Connect::RoutingProfile",
    "AWS::Connect::Rule",
    "AWS::Connect::SecurityKey",
    "AWS::Connect::SecurityProfile",
    "AWS::Connect::TaskTemplate",
    "AWS::Connect::TrafficDistributionGroup",
    "AWS::Connect::User",
    "AWS::Connect::UserHierarchyGroup",
    "AWS::Connect::View",
    "AWS::Connect::ViewVersion",
    "AWS::ConnectCampaigns::Campaign",
    "AWS::ControlTower::EnabledBaseline",
    "AWS::ControlTower::EnabledControl",
    "AWS::ControlTower::LandingZone",
    "AWS::CustomerProfiles::CalculatedAttributeDefinition",
    "AWS::CustomerProfiles::Domain",
    "AWS::CustomerProfiles::EventStream",
    "AWS::CustomerProfiles::Integration",
    "AWS::CustomerProfiles::ObjectType",
    "AWS::DAX::Cluster",
    "AWS::DAX::ParameterGroup",
    "AWS::DAX::SubnetGroup",
    "AWS::DLM::LifecyclePolicy",
    "AWS::DMS::Certificate",
    "AWS::DMS::DataProvider",
    "AWS::DMS::Endpoint",
    "AWS::DMS::EventSubscription",
    "AWS::DMS::InstanceProfile",
    "AWS::DMS::MigrationProject",
    "AWS::DMS::ReplicationConfig",
    "AWS::DMS::ReplicationInstance",
    "AWS::DMS::ReplicationSubnetGroup",
    "AWS::DMS::ReplicationTask",
    "AWS::DataBrew::Dataset",
    "AWS::DataBrew::Job",
    "AWS::DataBrew::Project",
    "AWS::DataBrew::Recipe",
    "AWS::DataBrew::Ruleset",
    "AWS::DataBrew::Schedule",
    "AWS::DataPipeline::Pipeline",
    "AWS::DataSync::Agent",
    "AWS::DataSync::LocationAzureBlob",
    "AWS::DataSync::LocationEFS",
    "AWS::DataSync::LocationFSxLustre",
    "AWS::DataSync::LocationFSxONTAP",
    "AWS::DataSync::LocationFSxOpenZFS",
    "AWS::DataSync::LocationFSxWindows",
    "AWS::DataSync::LocationHDFS",
    "AWS::DataSync::LocationNFS",
    "AWS::DataSync::LocationObjectStorage",
    "AWS::DataSync::LocationS3",
    "AWS::DataSync::LocationSMB",
    "AWS::DataSync::StorageSystem",
    "AWS::DataSync::Task",
    "AWS::DataZone::DataSource",
    "AWS::DataZone::Domain",
    "AWS::DataZone::Environment",
    "AWS::DataZone::EnvironmentBlueprintConfiguration",
    "AWS::DataZone::EnvironmentProfile",
    "AWS::DataZone::GroupProfile",
    "AWS::DataZone::Project",
    "AWS::DataZone::ProjectMembership",
    "AWS::DataZone::SubscriptionTarget",
    "AWS::DataZone::UserProfile",
    "AWS::Deadline::Farm",
    "AWS::Deadline::Fleet",
    "AWS::Deadline::LicenseEndpoint",
    "AWS::Deadline::MeteredProduct",
    "AWS::Deadline::Monitor",
    "AWS::Deadline::Queue",
    "AWS::Deadline::QueueEnvironment",
    "AWS::Deadline::QueueFleetAssociation",
    "AWS::Deadline::StorageProfile",
    "AWS::Detective::Graph",
    "AWS::Detective::MemberInvitation",
    "AWS::Detective::OrganizationAdmin",
    "AWS::DevOpsGuru::LogAnomalyDetectionIntegration",
    "AWS::DevOpsGuru::NotificationChannel",
    "AWS::DevOpsGuru::ResourceCollection",
    "AWS::DirectoryService::MicrosoftAD",
    "AWS::DirectoryService::SimpleAD",
    "AWS::DocDB::DBCluster",
    "AWS::DocDB::DBClusterParameterGroup",
    "AWS::DocDB::DBInstance",
    "AWS::DocDB::DBSubnetGroup",
    "AWS::DocDB::EventSubscription",
    "AWS::DocDBElastic::Cluster",
    "AWS::DynamoDB::GlobalTable",
    "AWS::DynamoDB::Table",
    "AWS::EC2::CapacityReservation",
    "AWS::EC2::CapacityReservationFleet",
    "AWS::EC2::CarrierGateway",
    "AWS::EC2::ClientVpnAuthorizationRule",
    "AWS::EC2::ClientVpnEndpoint",
    "AWS::EC2::ClientVpnRoute",
    "AWS::EC2::ClientVpnTargetNetworkAssociation",
    "AWS::EC2::CustomerGateway",
    "AWS::EC2::DHCPOptions",
    "AWS::EC2::EC2Fleet",
    "AWS::EC2::EIP",
    "AWS::EC2::EIPAssociation",
    "AWS::EC2::EgressOnlyInternetGateway",
    "AWS::EC2::EnclaveCertificateIamRoleAssociation",
    "AWS::EC2::FlowLog",
    "AWS::EC2::GatewayRouteTableAssociation",
    "AWS::EC2::Host",
    "AWS::EC2::IPAM",
    "AWS::EC2::IPAMAllocation",
    "AWS::EC2::IPAMPool",
    "AWS::EC2::IPAMPoolCidr",
    "AWS::EC2::IPAMResourceDiscovery",
    "AWS::EC2::IPAMResourceDiscoveryAssociation",
    "AWS::EC2::IPAMScope",
    "AWS::EC2::Instance",
    "AWS::EC2::InstanceConnectEndpoint",
    "AWS::EC2::InternetGateway",
    "AWS::EC2::KeyPair",
    "AWS::EC2::LaunchTemplate",
    "AWS::EC2::LocalGatewayRoute",
    "AWS::EC2::LocalGatewayRouteTable",
    "AWS::EC2::LocalGatewayRouteTableVPCAssociation",
    "AWS::EC2::LocalGatewayRouteTableVirtualInterfaceGroupAssociation",
    "AWS::EC2::NatGateway",
    "AWS::EC2::NetworkAcl",
    "AWS::EC2::NetworkAclEntry",
    "AWS::EC2::NetworkInsightsAccessScope",
    "AWS::EC2::NetworkInsightsAccessScopeAnalysis",
    "AWS::EC2::NetworkInsightsAnalysis",
    "AWS::EC2::NetworkInsightsPath",
    "AWS::EC2::NetworkInterface",
    "AWS::EC2::NetworkInterfaceAttachment",
    "AWS::EC2::NetworkInterfacePermission",
    "AWS::EC2::NetworkPerformanceMetricSubscription",
    "AWS::EC2::PlacementGroup",
    "AWS::EC2::PrefixList",
    "AWS::EC2::Route",
    "AWS::EC2::RouteTable",
    "AWS::EC2::SecurityGroup",
    "AWS::EC2::SecurityGroupEgress",
    "AWS::EC2::SecurityGroupIngress",
    "AWS::EC2::SnapshotBlockPublicAccess",
    "AWS::EC2::SpotFleet",
    "AWS::EC2::Subnet",
    "AWS::EC2::SubnetCidrBlock",
    "AWS::EC2::SubnetNetworkAclAssociation",
    "AWS::EC2::SubnetRouteTableAssociation",
    "AWS::EC2::TrafficMirrorFilter",
    "AWS::EC2::TrafficMirrorFilterRule",
    "AWS::EC2::TrafficMirrorSession",
    "AWS::EC2::TrafficMirrorTarget",
    "AWS::EC2::TransitGateway",
    "AWS::EC2::TransitGatewayAttachment",
    "AWS::EC2::TransitGatewayConnect",
    "AWS::EC2::TransitGatewayMulticastDomain",
    "AWS::EC2::TransitGatewayMulticastDomainAssociation",
    "AWS::EC2::TransitGatewayMulticastGroupMember",
    "AWS::EC2::TransitGatewayMulticastGroupSource",
    "AWS::EC2::TransitGatewayPeeringAttachment",
    "AWS::EC2::TransitGatewayRoute",
    "AWS::EC2::TransitGatewayRouteTable",
    "AWS::EC2::TransitGatewayRouteTableAssociation",
    "AWS::EC2::TransitGatewayRouteTablePropagation",
    "AWS::EC2::TransitGatewayVpcAttachment",
    "AWS::EC2::VPC",
    "AWS::EC2::VPCCidrBlock",
    "AWS::EC2::VPCDHCPOptionsAssociation",
    "AWS::EC2::VPCEndpoint",
    "AWS::EC2::VPCEndpointConnectionNotification",
    "AWS::EC2::VPCEndpointService",
    "AWS::EC2::VPCEndpointServicePermissions",
    "AWS::EC2::VPCGatewayAttachment",
    "AWS::EC2::VPCPeeringConnection",
    "AWS::EC2::VPNConnection",
    "AWS::EC2::VPNConnectionRoute",
    "AWS::EC2::VPNGateway",
    "AWS::EC2::VPNGatewayRoutePropagation",
    "AWS::EC2::VerifiedAccessEndpoint",
    "AWS::EC2::VerifiedAccessGroup",
    "AWS::EC2::VerifiedAccessInstance",
    "AWS::EC2::VerifiedAccessTrustProvider",
    "AWS::EC2::Volume",
    "AWS::EC2::VolumeAttachment",
    "AWS::ECR::PublicRepository",
    "AWS::ECR::PullThroughCacheRule",
    "AWS::ECR::RegistryPolicy",
    "AWS::ECR::ReplicationConfiguration",
    "AWS::ECR::Repository",
    "AWS::ECR::RepositoryCreationTemplate",
    "AWS::ECS::CapacityProvider",
    "AWS::ECS::Cluster",
    "AWS::ECS::ClusterCapacityProviderAssociations",
    "AWS::ECS::PrimaryTaskSet",
    "AWS::ECS::Service",
    "AWS::ECS::TaskDefinition",
    "AWS::ECS::TaskSet",
    "AWS::EFS::AccessPoint",
    "AWS::EFS::FileSystem",
    "AWS::EFS::MountTarget",
    "AWS::EKS::AccessEntry",
    "AWS::EKS::Addon",
    "AWS::EKS::Cluster",
    "AWS::EKS::FargateProfile",
    "AWS::EKS::IdentityProviderConfig",
    "AWS::EKS::Nodegroup",
    "AWS::EKS::PodIdentityAssociation",
    "AWS::EMR::Cluster",
    "AWS::EMR::InstanceFleetConfig",
    "AWS::EMR::InstanceGroupConfig",
    "AWS::EMR::SecurityConfiguration",
    "AWS::EMR::Step",
    "AWS::EMR::Studio",
    "AWS::EMR::StudioSessionMapping",
    "AWS::EMR::WALWorkspace",
    "AWS::EMRContainers::VirtualCluster",
    "AWS::EMRServerless::Application",
    "AWS::ElastiCache::CacheCluster",
    "AWS::ElastiCache::GlobalReplicationGroup",
    "AWS::ElastiCache::ParameterGroup",
    "AWS::ElastiCache::ReplicationGroup",
    "AWS::ElastiCache::SecurityGroup",
    "AWS::ElastiCache::SecurityGroupIngress",
    "AWS::ElastiCache::ServerlessCache",
    "AWS::ElastiCache::SubnetGroup",
    "AWS::ElastiCache::User",
    "AWS::ElastiCache::UserGroup",
    "AWS::ElasticBeanstalk::Application",
    "AWS::ElasticBeanstalk::ApplicationVersion",
    "AWS::ElasticBeanstalk::ConfigurationTemplate",
    "AWS::ElasticBeanstalk::Environment",
    "AWS::ElasticLoadBalancing::LoadBalancer",
    "AWS::ElasticLoadBalancingV2::Listener",
    "AWS::ElasticLoadBalancingV2::ListenerCertificate",
    "AWS::ElasticLoadBalancingV2::ListenerRule",
    "AWS::ElasticLoadBalancingV2::LoadBalancer",
    "AWS::ElasticLoadBalancingV2::TargetGroup",
    "AWS::ElasticLoadBalancingV2::TrustStore",
    "AWS::ElasticLoadBalancingV2::TrustStoreRevocation",
    "AWS::Elasticsearch::Domain",
    "AWS::EntityResolution::IdMappingWorkflow",
    "AWS::EntityResolution::IdNamespace",
    "AWS::EntityResolution::MatchingWorkflow",
    "AWS::EntityResolution::PolicyStatement",
    "AWS::EntityResolution::SchemaMapping",
    "AWS::EventSchemas::Discoverer",
    "AWS::EventSchemas::Registry",
    "AWS::EventSchemas::RegistryPolicy",
    "AWS::EventSchemas::Schema",
    "AWS::Events::ApiDestination",
    "AWS::Events::Archive",
    "AWS::Events::Connection",
    "AWS::Events::Endpoint",
    "AWS::Events::EventBus",
    "AWS::Events::EventBusPolicy",
    "AWS::Events::Rule",
    "AWS::Evidently::Experiment",
    "AWS::Evidently::Feature",
    "AWS::Evidently::Launch",
    "AWS::Evidently::Project",
    "AWS::Evidently::Segment",
    "AWS::FIS::ExperimentTemplate",
    "AWS::FIS::TargetAccountConfiguration",
    "AWS::FMS::NotificationChannel",
    "AWS::FMS::Policy",
    "AWS::FMS::ResourceSet",
    "AWS::FSx::DataRepositoryAssociation",
    "AWS::FSx::FileSystem",
    "AWS::FSx::Snapshot",
    "AWS::FSx::StorageVirtualMachine",
    "AWS::FSx::Volume",
    "AWS::FinSpace::Environment",
    "AWS::Forecast::Dataset",
    "AWS::Forecast::DatasetGroup",
    "AWS::FraudDetector::Detector",
    "AWS::FraudDetector::EntityType",
    "AWS::FraudDetector::EventType",
    "AWS::FraudDetector::Label",
    "AWS::FraudDetector::List",
    "AWS::FraudDetector::Outcome",
    "AWS::FraudDetector::Variable",
    "AWS::GameLift::Alias",
    "AWS::GameLift::Build",
    "AWS::GameLift::ContainerGroupDefinition",
    "AWS::GameLift::Fleet",
    "AWS::GameLift::GameServerGroup",
    "AWS::GameLift::GameSessionQueue",
    "AWS::GameLift::Location",
    "AWS::GameLift::MatchmakingConfiguration",
    "AWS::GameLift::MatchmakingRuleSet",
    "AWS::GameLift::Script",
    "AWS::GlobalAccelerator::Accelerator",
    "AWS::GlobalAccelerator::CrossAccountAttachment",
    "AWS::GlobalAccelerator::EndpointGroup",
    "AWS::GlobalAccelerator::Listener",
    "AWS::Glue::Classifier",
    "AWS::Glue::Connection",
    "AWS::Glue::Crawler",
    "AWS::Glue::CustomEntityType",
    "AWS::Glue::DataCatalogEncryptionSettings",
    "AWS::Glue::DataQualityRuleset",
    "AWS::Glue::Database",
    "AWS::Glue::DevEndpoint",
    "AWS::Glue::Job",
    "AWS::Glue::MLTransform",
    "AWS::Glue::Partition",
    "AWS::Glue::Registry",
    "AWS::Glue::Schema",
    "AWS::Glue::SchemaVersion",
    "AWS::Glue::SchemaVersionMetadata",
    "AWS::Glue::SecurityConfiguration",
    "AWS::Glue::Table",
    "AWS::Glue::TableOptimizer",
    "AWS::Glue::Trigger",
    "AWS::Glue::Workflow",
    "AWS::Grafana::Workspace",
    "AWS::Greengrass::ConnectorDefinition",
    "AWS::Greengrass::ConnectorDefinitionVersion",
    "AWS::Greengrass::CoreDefinition",
    "AWS::Greengrass::CoreDefinitionVersion",
    "AWS::Greengrass::DeviceDefinition",
    "AWS::Greengrass::DeviceDefinitionVersion",
    "AWS::Greengrass::FunctionDefinition",
    "AWS::Greengrass::FunctionDefinitionVersion",
    "AWS::Greengrass::Group",
    "AWS::Greengrass::GroupVersion",
    "AWS::Greengrass::LoggerDefinition",
    "AWS::Greengrass::LoggerDefinitionVersion",
    "AWS::Greengrass::ResourceDefinition",
    "AWS::Greengrass::ResourceDefinitionVersion",
    "AWS::Greengrass::SubscriptionDefinition",
    "AWS::Greengrass::SubscriptionDefinitionVersion",
    "AWS::GreengrassV2::ComponentVersion",
    "AWS::GreengrassV2::Deployment",
    "AWS::GroundStation::Config",
    "AWS::GroundStation::DataflowEndpointGroup",
    "AWS::GroundStation::MissionProfile",
    "AWS::GuardDuty::Detector",
    "AWS::GuardDuty::Filter",
    "AWS::GuardDuty::IPSet",
    "AWS::GuardDuty::MalwareProtectionPlan",
    "AWS::GuardDuty::Master",
    "AWS::GuardDuty::Member",
    "AWS::GuardDuty::ThreatIntelSet",
    "AWS::HealthImaging::Datastore",
    "AWS::HealthLake::FHIRDatastore",
    "AWS::IAM::AccessKey",
    "AWS::IAM::Group",
    "AWS::IAM::GroupPolicy",
    "AWS::IAM::InstanceProfile",
    "AWS::IAM::ManagedPolicy",
    "AWS::IAM::OIDCProvider",
    "AWS::IAM::Policy",
    "AWS::IAM::Role",
    "AWS::IAM::RolePolicy",
    "AWS::IAM::SAMLProvider",
    "AWS::IAM::ServerCertificate",
    "AWS::IAM::ServiceLinkedRole",
    "AWS::IAM::User",
    "AWS::IAM::UserPolicy",
    "AWS::IAM::UserToGroupAddition",
    "AWS::IAM::VirtualMFADevice",
    "AWS::IVS::Channel",
    "AWS::IVS::EncoderConfiguration",
    "AWS::IVS::PlaybackKeyPair",
    "AWS::IVS::PlaybackRestrictionPolicy",
    "AWS::IVS::RecordingConfiguration",
    "AWS::IVS::Stage",
    "AWS::IVS::StorageConfiguration",
    "AWS::IVS::StreamKey",
    "AWS::IVSChat::LoggingConfiguration",
    "AWS::IVSChat::Room",
    "AWS::IdentityStore::Group",
    "AWS::IdentityStore::GroupMembership",
    "AWS::ImageBuilder::Component",
    "AWS::ImageBuilder::ContainerRecipe",
    "AWS::ImageBuilder::DistributionConfiguration",
    "AWS::ImageBuilder::Image",
    "AWS::ImageBuilder::ImagePipeline",
    "AWS::ImageBuilder::ImageRecipe",
    "AWS::ImageBuilder::InfrastructureConfiguration",
    "AWS::ImageBuilder::LifecyclePolicy",
    "AWS::ImageBuilder::Workflow",
    "AWS::Inspector::AssessmentTarget",
    "AWS::Inspector::AssessmentTemplate",
    "AWS::Inspector::ResourceGroup",
    "AWS::InspectorV2::CisScanConfiguration",
    "AWS::InspectorV2::Filter",
    "AWS::InternetMonitor::Monitor",
    "AWS::IoT1Click::Device",
    "AWS::IoT1Click::Placement",
    "AWS::IoT1Click::Project",
    "AWS::IoT::AccountAuditConfiguration",
    "AWS::IoT::Authorizer",
    "AWS::IoT::BillingGroup",
    "AWS::IoT::CACertificate",
    "AWS::IoT::Certificate",
    "AWS::IoT::CertificateProvider",
    "AWS::IoT::CustomMetric",
    "AWS::IoT::Dimension",
    "AWS::IoT::DomainConfiguration",
    "AWS::IoT::FleetMetric",
    "AWS::IoT::JobTemplate",
    "AWS::IoT::Logging",
    "AWS::IoT::MitigationAction",
    "AWS::IoT::Policy",
    "AWS::IoT::PolicyPrincipalAttachment",
    "AWS::IoT::ProvisioningTemplate",
    "AWS::IoT::ResourceSpecificLogging",
    "AWS::IoT::RoleAlias",
    "AWS::IoT::ScheduledAudit",
    "AWS::IoT::SecurityProfile",
    "AWS::IoT::SoftwarePackage",
    "AWS::IoT::SoftwarePackageVersion",
    "AWS::IoT::Thing",
    "AWS::IoT::ThingGroup",
    "AWS::IoT::ThingPrincipalAttachment",
    "AWS::IoT::ThingType",
    "AWS::IoT::TopicRule",
    "AWS::IoT::TopicRuleDestination",
    "AWS::IoTAnalytics::Channel",
    "AWS::IoTAnalytics::Dataset",
    "AWS::IoTAnalytics::Datastore",
    "AWS::IoTAnalytics::Pipeline",
    "AWS::IoTCoreDeviceAdvisor::SuiteDefinition",
    "AWS::IoTEvents::AlarmModel",
    "AWS::IoTEvents::DetectorModel",
    "AWS::IoTEvents::Input",
    "AWS::IoTFleetHub::Application",
    "AWS::IoTFleetWise::Campaign",
    "AWS::IoTFleetWise::DecoderManifest",
    "AWS::IoTFleetWise::Fleet",
    "AWS::IoTFleetWise::ModelManifest",
    "AWS::IoTFleetWise::SignalCatalog",
    "AWS::IoTFleetWise::Vehicle",
    "AWS::IoTSiteWise::AccessPolicy",
    "AWS::IoTSiteWise::Asset",
    "AWS::IoTSiteWise::AssetModel",
    "AWS::IoTSiteWise::Dashboard",
    "AWS::IoTSiteWise::Gateway",
    "AWS::IoTSiteWise::Portal",
    "AWS::IoTSiteWise::Project",
    "AWS::IoTThingsGraph::FlowTemplate",
    "AWS::IoTTwinMaker::ComponentType",
    "AWS::IoTTwinMaker::Entity",
    "AWS::IoTTwinMaker::Scene",
    "AWS::IoTTwinMaker::SyncJob",
    "AWS::IoTTwinMaker::Workspace",
    "AWS::IoTWireless::Destination",
    "AWS::IoTWireless::DeviceProfile",
    "AWS::IoTWireless::FuotaTask",
    "AWS::IoTWireless::MulticastGroup",
    "AWS::IoTWireless::NetworkAnalyzerConfiguration",
    "AWS::IoTWireless::PartnerAccount",
    "AWS::IoTWireless::ServiceProfile",
    "AWS::IoTWireless::TaskDefinition",
    "AWS::IoTWireless::WirelessDevice",
    "AWS::IoTWireless::WirelessDeviceImportTask",
    "AWS::IoTWireless::WirelessGateway",
    "AWS::KMS::Alias",
    "AWS::KMS::Key",
    "AWS::KMS::ReplicaKey",
    "AWS::KafkaConnect::Connector",
    "AWS::KafkaConnect::CustomPlugin",
    "AWS::KafkaConnect::WorkerConfiguration",
    "AWS::Kendra::DataSource",
    "AWS::Kendra::Faq",
    "AWS::Kendra::Index",
    "AWS::KendraRanking::ExecutionPlan",
    "AWS::Kinesis::Stream",
    "AWS::Kinesis::StreamConsumer",
    "AWS::KinesisAnalytics::Application",
    "AWS::KinesisAnalytics::ApplicationOutput",
    "AWS::KinesisAnalytics::ApplicationReferenceDataSource",
    "AWS::KinesisAnalyticsV2::Application",
    "AWS::KinesisAnalyticsV2::ApplicationCloudWatchLoggingOption",
    "AWS::KinesisAnalyticsV2::ApplicationOutput",
    "AWS::KinesisAnalyticsV2::ApplicationReferenceDataSource",
    "AWS::KinesisFirehose::DeliveryStream",
    "AWS::KinesisVideo::SignalingChannel",
    "AWS::KinesisVideo::Stream",
    "AWS::LakeFormation::DataCellsFilter",
    "AWS::LakeFormation::DataLakeSettings",
    "AWS::LakeFormation::Permissions",
    "AWS::LakeFormation::PrincipalPermissions",
    "AWS::LakeFormation::Resource",
    "AWS::LakeFormation::Tag",
    "AWS::LakeFormation::TagAssociation",
    "AWS::Lambda::Alias",
    "AWS::Lambda::CodeSigningConfig",
    "AWS::Lambda::EventInvokeConfig",
    "AWS::Lambda::EventSourceMapping",
    "AWS::Lambda::Function",
    "AWS::Lambda::LayerVersion",
    "AWS::Lambda::LayerVersionPermission",
    "AWS::Lambda::Permission",
    "AWS::Lambda::Url",
    "AWS::Lambda::Version",
    "AWS::LaunchWizard::Deployment",
    "AWS::Lex::Bot",
    "AWS::Lex::BotAlias",
    "AWS::Lex::BotVersion",
    "AWS::Lex::ResourcePolicy",
    "AWS::LicenseManager::Grant",
    "AWS::LicenseManager::License",
    "AWS::Lightsail::Alarm",
    "AWS::Lightsail::Bucket",
    "AWS::Lightsail::Certificate",
    "AWS::Lightsail::Container",
    "AWS::Lightsail::Database",
    "AWS::Lightsail::Disk",
    "AWS::Lightsail::Distribution",
    "AWS::Lightsail::Instance",
    "AWS::Lightsail::LoadBalancer",
    "AWS::Lightsail::LoadBalancerTlsCertificate",
    "AWS::Lightsail::StaticIp",
    "AWS::Location::APIKey",
    "AWS::Location::GeofenceCollection",
    "AWS::Location::Map",
    "AWS::Location::PlaceIndex",
    "AWS::Location::RouteCalculator",
    "AWS::Location::Tracker",
    "AWS::Location::TrackerConsumer",
    "AWS::Logs::AccountPolicy",
    "AWS::Logs::Delivery",
    "AWS::Logs::DeliveryDestination",
    "AWS::Logs::DeliverySource",
    "AWS::Logs::Destination",
    "AWS::Logs::LogAnomalyDetector",
    "AWS::Logs::LogGroup",
    "AWS::Logs::LogStream",
    "AWS::Logs::MetricFilter",
    "AWS::Logs::QueryDefinition",
    "AWS::Logs::ResourcePolicy",
    "AWS::Logs::SubscriptionFilter",
    "AWS::LookoutEquipment::InferenceScheduler",
    "AWS::LookoutMetrics::Alert",
    "AWS::LookoutMetrics::AnomalyDetector",
    "AWS::LookoutVision::Project",
    "AWS::M2::Application",
    "AWS::M2::Environment",
    "AWS::MSK::BatchScramSecret",
    "AWS::MSK::Cluster",
    "AWS::MSK::ClusterPolicy",
    "AWS::MSK::Configuration",
    "AWS::MSK::Replicator",
    "AWS::MSK::ServerlessCluster",
    "AWS::MSK::VpcConnection",
    "AWS::MWAA::Environment",
    "AWS::Macie::AllowList",
    "AWS::Macie::CustomDataIdentifier",
    "AWS::Macie::FindingsFilter",
    "AWS::Macie::Session",
    "AWS::ManagedBlockchain::Accessor",
    "AWS::ManagedBlockchain::Member",
    "AWS::ManagedBlockchain::Node",
    "AWS::MediaConnect::Bridge",
    "AWS::MediaConnect::BridgeOutput",
    "AWS::MediaConnect::BridgeSource",
    "AWS::MediaConnect::Flow",
    "AWS::MediaConnect::FlowEntitlement",
    "AWS::MediaConnect::FlowOutput",
    "AWS::MediaConnect::FlowSource",
    "AWS::MediaConnect::FlowVpcInterface",
    "AWS::MediaConnect::Gateway",
    "AWS::MediaConvert::JobTemplate",
    "AWS::MediaConvert::Preset",
    "AWS::MediaConvert::Queue",
    "AWS::MediaLive::Channel",
    "AWS::MediaLive::Input",
    "AWS::MediaLive::InputSecurityGroup",
    "AWS::MediaLive::Multiplex",
    "AWS::MediaLive::Multiplexprogram",
    "AWS::MediaPackage::Asset",
    "AWS::MediaPackage::Channel",
    "AWS::MediaPackage::OriginEndpoint",
    "AWS::MediaPackage::PackagingConfiguration",
    "AWS::MediaPackage::PackagingGroup",
    "AWS::MediaPackageV2::Channel",
    "AWS::MediaPackageV2::ChannelGroup",
    "AWS::MediaPackageV2::ChannelPolicy",
    "AWS::MediaPackageV2::OriginEndpoint",
    "AWS::MediaPackageV2::OriginEndpointPolicy",
    "AWS::MediaStore::Container",
    "AWS::MediaTailor::Channel",
    "AWS::MediaTailor::ChannelPolicy",
    "AWS::MediaTailor::LiveSource",
    "AWS::MediaTailor::PlaybackConfiguration",
    "AWS::MediaTailor::SourceLocation",
    "AWS::MediaTailor::VodSource",
    "AWS::MemoryDB::ACL",
    "AWS::MemoryDB::Cluster",
    "AWS::MemoryDB::ParameterGroup",
    "AWS::MemoryDB::SubnetGroup",
    "AWS::MemoryDB::User",
    "AWS::Neptune::DBCluster",
    "AWS::Neptune::DBClusterParameterGroup",
    "AWS::Neptune::DBInstance",
    "AWS::Neptune::DBParameterGroup",
    "AWS::Neptune::DBSubnetGroup",
    "AWS::Neptune::EventSubscription",
    "AWS::NeptuneGraph::Graph",
    "AWS::NeptuneGraph::PrivateGraphEndpoint",
    "AWS::NetworkFirewall::Firewall",
    "AWS::NetworkFirewall::FirewallPolicy",
    "AWS::NetworkFirewall::LoggingConfiguration",
    "AWS::NetworkFirewall::RuleGroup",
    "AWS::NetworkFirewall::TLSInspectionConfiguration",
    "AWS::NetworkManager::ConnectAttachment",
    "AWS::NetworkManager::ConnectPeer",
    "AWS::NetworkManager::CoreNetwork",
    "AWS::NetworkManager::CustomerGatewayAssociation",
    "AWS::NetworkManager::Device",
    "AWS::NetworkManager::GlobalNetwork",
    "AWS::NetworkManager::Link",
    "AWS::NetworkManager::LinkAssociation",
    "AWS::NetworkManager::Site",
    "AWS::NetworkManager::SiteToSiteVpnAttachment",
    "AWS::NetworkManager::TransitGatewayPeering",
    "AWS::NetworkManager::TransitGatewayRegistration",
    "AWS::NetworkManager::TransitGatewayRouteTableAttachment",
    "AWS::NetworkManager::VpcAttachment",
    "AWS::NimbleStudio::LaunchProfile",
    "AWS::NimbleStudio::StreamingImage",
    "AWS::NimbleStudio::Studio",
    "AWS::NimbleStudio::StudioComponent",
    "AWS::OSIS::Pipeline",
    "AWS::Oam::Link",
    "AWS::Oam::Sink",
    "AWS::Omics::AnnotationStore",
    "AWS::Omics::ReferenceStore",
    "AWS::Omics::RunGroup",
    "AWS::Omics::SequenceStore",
    "AWS::Omics::VariantStore",
    "AWS::Omics::Workflow",
    "AWS::OpenSearchServerless::AccessPolicy",
    "AWS::OpenSearchServerless::Collection",
    "AWS::OpenSearchServerless::LifecyclePolicy",
    "AWS::OpenSearchServerless::SecurityConfig",
    "AWS::OpenSearchServerless::SecurityPolicy",
    "AWS::OpenSearchServerless::VpcEndpoint",
    "AWS::OpenSearchService::Domain",
    "AWS::OpsWorks::App",
    "AWS::OpsWorks::ElasticLoadBalancerAttachment",
    "AWS::OpsWorks::Instance",
    "AWS::OpsWorks::Layer",
    "AWS::OpsWorks::Stack",
    "AWS::OpsWorks::UserProfile",
    "AWS::OpsWorks::Volume",
    "AWS::OpsWorksCM::Server",
    "AWS::Organizations::Account",
    "AWS::Organizations::Organization",
    "AWS::Organizations::OrganizationalUnit",
    "AWS::Organizations::Policy",
    "AWS::Organizations::ResourcePolicy",
    "AWS::PCAConnectorAD::Connector",
    "AWS::PCAConnectorAD::DirectoryRegistration",
    "AWS::PCAConnectorAD::ServicePrincipalName",
    "AWS::PCAConnectorAD::Template",
    "AWS::PCAConnectorAD::TemplateGroupAccessControlEntry",
    "AWS::Panorama::ApplicationInstance",
    "AWS::Panorama::Package",
    "AWS::Panorama::PackageVersion",
    "AWS::PaymentCryptography::Alias",
    "AWS::PaymentCryptography::Key",
    "AWS::Personalize::Dataset",
    "AWS::Personalize::DatasetGroup",
    "AWS::Personalize::Schema",
    "AWS::Personalize::Solution",
    "AWS::Pinpoint::ADMChannel",
    "AWS::Pinpoint::APNSChannel",
    "AWS::Pinpoint::APNSSandboxChannel",
    "AWS::Pinpoint::APNSVoipChannel",
    "AWS::Pinpoint::APNSVoipSandboxChannel",
    "AWS::Pinpoint::App",
    "AWS::Pinpoint::ApplicationSettings",
    "AWS::Pinpoint::BaiduChannel",
    "AWS::Pinpoint::Campaign",
    "AWS::Pinpoint::EmailChannel",
    "AWS::Pinpoint::EmailTemplate",
    "AWS::Pinpoint::EventStream",
    "AWS::Pinpoint::GCMChannel",
    "AWS::Pinpoint::InAppTemplate",
    "AWS::Pinpoint::PushTemplate",
    "AWS::Pinpoint::SMSChannel",
    "AWS::Pinpoint::Segment",
    "AWS::Pinpoint::SmsTemplate",
    "AWS::Pinpoint::VoiceChannel",
    "AWS::PinpointEmail::ConfigurationSet",
    "AWS::PinpointEmail::ConfigurationSetEventDestination",
    "AWS::PinpointEmail::DedicatedIpPool",
    "AWS::PinpointEmail::Identity",
    "AWS::Pipes::Pipe",
    "AWS::Proton::EnvironmentAccountConnection",
    "AWS::Proton::EnvironmentTemplate",
    "AWS::Proton::ServiceTemplate",
    "AWS::QBusiness::Application",
    "AWS::QBusiness::DataSource",
    "AWS::QBusiness::Index",
    "AWS::QBusiness::Plugin",
    "AWS::QBusiness::Retriever",
    "AWS::QBusiness::WebExperience",
    "AWS::QLDB::Ledger",
    "AWS::QLDB::Stream",
    "AWS::QuickSight::Analysis",
    "AWS::QuickSight::Dashboard",
    "AWS::QuickSight::DataSet",
    "AWS::QuickSight::DataSource",
    "AWS::QuickSight::RefreshSchedule",
    "AWS::QuickSight::Template",
    "AWS::QuickSight::Theme",
    "AWS::QuickSight::Topic",
    "AWS::QuickSight::VPCConnection",
    "AWS::RAM::Permission",
    "AWS::RAM::ResourceShare",
    "AWS::RDS::CustomDBEngineVersion",
    "AWS::RDS::DBCluster",
    "AWS::RDS::DBClusterParameterGroup",
    "AWS::RDS::DBInstance",
    "AWS::RDS::DBParameterGroup",
    "AWS::RDS::DBProxy",
    "AWS::RDS::DBProxyEndpoint",
    "AWS::RDS::DBProxyTargetGroup",
    "AWS::RDS::DBSecurityGroup",
    "AWS::RDS::DBSecurityGroupIngress",
    "AWS::RDS::DBSubnetGroup",
    "AWS::RDS::EventSubscription",
    "AWS::RDS::GlobalCluster",
    "AWS::RDS::Integration",
    "AWS::RDS::OptionGroup",
    "AWS::RUM::AppMonitor",
    "AWS::Redshift::Cluster",
    "AWS::Redshift::ClusterParameterGroup",
    "AWS::Redshift::ClusterSecurityGroup",
    "AWS::Redshift::ClusterSecurityGroupIngress",
    "AWS::Redshift::ClusterSubnetGroup",
    "AWS::Redshift::EndpointAccess",
    "AWS::Redshift::EndpointAuthorization",
    "AWS::Redshift::EventSubscription",
    "AWS::Redshift::ScheduledAction",
    "AWS::RedshiftServerless::Namespace",
    "AWS::RedshiftServerless::Workgroup",
    "AWS::RefactorSpaces::Application",
    "AWS::RefactorSpaces::Environment",
    "AWS::RefactorSpaces::Route",
    "AWS::RefactorSpaces::Service",
    "AWS::Rekognition::Collection",
    "AWS::Rekognition::Project",
    "AWS::Rekognition::StreamProcessor",
    "AWS::ResilienceHub::App",
    "AWS::ResilienceHub::ResiliencyPolicy",
    "AWS::ResourceExplorer2::DefaultViewAssociation",
    "AWS::ResourceExplorer2::Index",
    "AWS::ResourceExplorer2::View",
    "AWS::ResourceGroups::Group",
    "AWS::RoboMaker::Fleet",
    "AWS::RoboMaker::Robot",
    "AWS::RoboMaker::RobotApplication",
    "AWS::RoboMaker::RobotApplicationVersion",
    "AWS::RoboMaker::SimulationApplication",
    "AWS::RoboMaker::SimulationApplicationVersion",
    "AWS::RolesAnywhere::CRL",
    "AWS::RolesAnywhere::Profile",
    "AWS::RolesAnywhere::TrustAnchor",
    "AWS::Route53::CidrCollection",
    "AWS::Route53::DNSSEC",
    "AWS::Route53::HealthCheck",
    "AWS::Route53::HostedZone",
    "AWS::Route53::KeySigningKey",
    "AWS::Route53::RecordSet",
    "AWS::Route53::RecordSetGroup",
    "AWS::Route53Profiles::Profile",
    "AWS::Route53Profiles::ProfileAssociation",
    "AWS::Route53Profiles::ProfileResourceAssociation",
    "AWS::Route53RecoveryControl::Cluster",
    "AWS::Route53RecoveryControl::ControlPanel",
    "AWS::Route53RecoveryControl::RoutingControl",
    "AWS::Route53RecoveryControl::SafetyRule",
    "AWS::Route53RecoveryReadiness::Cell",
    "AWS::Route53RecoveryReadiness::ReadinessCheck",
    "AWS::Route53RecoveryReadiness::RecoveryGroup",
    "AWS::Route53RecoveryReadiness::ResourceSet",
    "AWS::Route53Resolver::FirewallDomainList",
    "AWS::Route53Resolver::FirewallRuleGroup",
    "AWS::Route53Resolver::FirewallRuleGroupAssociation",
    "AWS::Route53Resolver::OutpostResolver",
    "AWS::Route53Resolver::ResolverConfig",
    "AWS::Route53Resolver::ResolverDNSSECConfig",
    "AWS::Route53Resolver::ResolverEndpoint",
    "AWS::Route53Resolver::ResolverQueryLoggingConfig",
    "AWS::Route53Resolver::ResolverQueryLoggingConfigAssociation",
    "AWS::Route53Resolver::ResolverRule",
    "AWS::Route53Resolver::ResolverRuleAssociation",
    "AWS::S3::AccessGrant",
    "AWS::S3::AccessGrantsInstance",
    "AWS::S3::AccessGrantsLocation",
    "AWS::S3::AccessPoint",
    "AWS::S3::Bucket",
    "AWS::S3::BucketPolicy",
    "AWS::S3::MultiRegionAccessPoint",
    "AWS::S3::MultiRegionAccessPointPolicy",
    "AWS::S3::StorageLens",
    "AWS::S3::StorageLensGroup",
    "AWS::S3Express::BucketPolicy",
    "AWS::S3Express::DirectoryBucket",
    "AWS::S3ObjectLambda::AccessPoint",
    "AWS::S3ObjectLambda::AccessPointPolicy",
    "AWS::S3Outposts::AccessPoint",
    "AWS::S3Outposts::Bucket",
    "AWS::S3Outposts::BucketPolicy",
    "AWS::S3Outposts::Endpoint",
    "AWS::SDB::Domain",
    "AWS::SES::ConfigurationSet",
    "AWS::SES::ConfigurationSetEventDestination",
    "AWS::SES::ContactList",
    "AWS::SES::DedicatedIpPool",
    "AWS::SES::EmailIdentity",
    "AWS::SES::MailManagerAddonInstance",
    "AWS::SES::MailManagerAddonSubscription",
    "AWS::SES::MailManagerArchive",
    "AWS::SES::MailManagerIngressPoint",
    "AWS::SES::MailManagerRelay",
    "AWS::SES::MailManagerRuleSet",
    "AWS::SES::MailManagerTrafficPolicy",
    "AWS::SES::ReceiptFilter",
    "AWS::SES::ReceiptRule",
    "AWS::SES::ReceiptRuleSet",
    "AWS::SES::Template",
    "AWS::SES::VdmAttributes",
    "AWS::SNS::Subscription",
    "AWS::SNS::Topic",
    "AWS::SNS::TopicInlinePolicy",
    "AWS::SNS::TopicPolicy",
    "AWS::SQS::Queue",
    "AWS::SQS::QueueInlinePolicy",
    "AWS::SQS::QueuePolicy",
    "AWS::SSM::Association",
    "AWS::SSM::Document",
    "AWS::SSM::MaintenanceWindow",
    "AWS::SSM::MaintenanceWindowTarget",
    "AWS::SSM::MaintenanceWindowTask",
    "AWS::SSM::Parameter",
    "AWS::SSM::PatchBaseline",
    "AWS::SSM::ResourceDataSync",
    "AWS::SSM::ResourcePolicy",
    "AWS::SSMContacts::Contact",
    "AWS::SSMContacts::ContactChannel",
    "AWS::SSMContacts::Plan",
    "AWS::SSMContacts::Rotation",
    "AWS::SSMIncidents::ReplicationSet",
    "AWS::SSMIncidents::ResponsePlan",
    "AWS::SSO::Application",
    "AWS::SSO::ApplicationAssignment",
    "AWS::SSO::Assignment",
    "AWS::SSO::Instance",
    "AWS::SSO::InstanceAccessControlAttributeConfiguration",
    "AWS::SSO::PermissionSet",
    "AWS::SageMaker::App",
    "AWS::SageMaker::AppImageConfig",
    "AWS::SageMaker::CodeRepository",
    "AWS::SageMaker::DataQualityJobDefinition",
    "AWS::SageMaker::Device",
    "AWS::SageMaker::DeviceFleet",
    "AWS::SageMaker::Domain",
    "AWS::SageMaker::Endpoint",
    "AWS::SageMaker::EndpointConfig",
    "AWS::SageMaker::FeatureGroup",
    "AWS::SageMaker::Image",
    "AWS::SageMaker::ImageVersion",
    "AWS::SageMaker::InferenceComponent",
    "AWS::SageMaker::InferenceExperiment",
    "AWS::SageMaker::Model",
    "AWS::SageMaker::ModelBiasJobDefinition",
    "AWS::SageMaker::ModelCard",
    "AWS::SageMaker::ModelExplainabilityJobDefinition",
    "AWS::SageMaker::ModelPackage",
    "AWS::SageMaker::ModelPackageGroup",
    "AWS::SageMaker::ModelQualityJobDefinition",
    "AWS::SageMaker::MonitoringSchedule",
    "AWS::SageMaker::NotebookInstance",
    "AWS::SageMaker::NotebookInstanceLifecycleConfig",
    "AWS::SageMaker::Pipeline",
    "AWS::SageMaker::Project",
    "AWS::SageMaker::Space",
    "AWS::SageMaker::UserProfile",
    "AWS::SageMaker::Workteam",
    "AWS::Scheduler::Schedule",
    "AWS::Scheduler::ScheduleGroup",
    "AWS::SecretsManager::ResourcePolicy",
    "AWS::SecretsManager::RotationSchedule",
    "AWS::SecretsManager::Secret",
    "AWS::SecretsManager::SecretTargetAttachment",
    "AWS::SecurityHub::AutomationRule",
    "AWS::SecurityHub::ConfigurationPolicy",
    "AWS::SecurityHub::DelegatedAdmin",
    "AWS::SecurityHub::FindingAggregator",
    "AWS::SecurityHub::Hub",
    "AWS::SecurityHub::Insight",
    "AWS::SecurityHub::OrganizationConfiguration",
    "AWS::SecurityHub::PolicyAssociation",
    "AWS::SecurityHub::ProductSubscription",
    "AWS::SecurityHub::SecurityControl",
    "AWS::SecurityHub::Standard",
    "AWS::SecurityLake::AwsLogSource",
    "AWS::SecurityLake::DataLake",
    "AWS::SecurityLake::Subscriber",
    "AWS::SecurityLake::SubscriberNotification",
    "AWS::ServiceCatalog::AcceptedPortfolioShare",
    "AWS::ServiceCatalog::CloudFormationProduct",
    "AWS::ServiceCatalog::CloudFormationProvisionedProduct",
    "AWS::ServiceCatalog::LaunchNotificationConstraint",
    "AWS::ServiceCatalog::LaunchRoleConstraint",
    "AWS::ServiceCatalog::LaunchTemplateConstraint",
    "AWS::ServiceCatalog::Portfolio",
    "AWS::ServiceCatalog::PortfolioPrincipalAssociation",
    "AWS::ServiceCatalog::PortfolioProductAssociation",
    "AWS::ServiceCatalog::PortfolioShare",
    "AWS::ServiceCatalog::ResourceUpdateConstraint",
    "AWS::ServiceCatalog::ServiceAction",
    "AWS::ServiceCatalog::ServiceActionAssociation",
    "AWS::ServiceCatalog::StackSetConstraint",
    "AWS::ServiceCatalog::TagOption",
    "AWS::ServiceCatalog::TagOptionAssociation",
    "AWS::ServiceCatalogAppRegistry::Application",
    "AWS::ServiceCatalogAppRegistry::AttributeGroup",
    "AWS::ServiceCatalogAppRegistry::AttributeGroupAssociation",
    "AWS::ServiceCatalogAppRegistry::ResourceAssociation",
    "AWS::ServiceDiscovery::HttpNamespace",
    "AWS::ServiceDiscovery::Instance",
    "AWS::ServiceDiscovery::PrivateDnsNamespace",
    "AWS::ServiceDiscovery::PublicDnsNamespace",
    "AWS::ServiceDiscovery::Service",
    "AWS::Shield::DRTAccess",
    "AWS::Shield::ProactiveEngagement",
    "AWS::Shield::Protection",
    "AWS::Shield::ProtectionGroup",
    "AWS::Signer::ProfilePermission",
    "AWS::Signer::SigningProfile",
    "AWS::SimSpaceWeaver::Simulation",
    "AWS::StepFunctions::Activity",
    "AWS::StepFunctions::StateMachine",
    "AWS::StepFunctions::StateMachineAlias",
    "AWS::StepFunctions::StateMachineVersion",
    "AWS::SupportApp::AccountAlias",
    "AWS::SupportApp::SlackChannelConfiguration",
    "AWS::SupportApp::SlackWorkspaceConfiguration",
    "AWS::Synthetics::Canary",
    "AWS::Synthetics::Group",
    "AWS::SystemsManagerSAP::Application",
    "AWS::Timestream::Database",
    "AWS::Timestream::InfluxDBInstance",
    "AWS::Timestream::ScheduledQuery",
    "AWS::Timestream::Table",
    "AWS::Transfer::Agreement",
    "AWS::Transfer::Certificate",
    "AWS::Transfer::Connector",
    "AWS::Transfer::Profile",
    "AWS::Transfer::Server",
    "AWS::Transfer::User",
    "AWS::Transfer::Workflow",
    "AWS::VerifiedPermissions::IdentitySource",
    "AWS::VerifiedPermissions::Policy",
    "AWS::VerifiedPermissions::PolicyStore",
    "AWS::VerifiedPermissions::PolicyTemplate",
    "AWS::VoiceID::Domain",
    "AWS::VpcLattice::AccessLogSubscription",
    "AWS::VpcLattice::AuthPolicy",
    "AWS::VpcLattice::Listener",
    "AWS::VpcLattice::ResourcePolicy",
    "AWS::VpcLattice::Rule",
    "AWS::VpcLattice::Service",
    "AWS::VpcLattice::ServiceNetwork",
    "AWS::VpcLattice::ServiceNetworkServiceAssociation",
    "AWS::VpcLattice::ServiceNetworkVpcAssociation",
    "AWS::VpcLattice::TargetGroup",
    "AWS::WAF::ByteMatchSet",
    "AWS::WAF::IPSet",
    "AWS::WAF::Rule",
    "AWS::WAF::SizeConstraintSet",
    "AWS::WAF::SqlInjectionMatchSet",
    "AWS::WAF::WebACL",
    "AWS::WAF::XssMatchSet",
    "AWS::WAFRegional::ByteMatchSet",
    "AWS::WAFRegional::GeoMatchSet",
    "AWS::WAFRegional::IPSet",
    "AWS::WAFRegional::RateBasedRule",
    "AWS::WAFRegional::RegexPatternSet",
    "AWS::WAFRegional::Rule",
    "AWS::WAFRegional::SizeConstraintSet",
    "AWS::WAFRegional::SqlInjectionMatchSet",
    "AWS::WAFRegional::WebACL",
    "AWS::WAFRegional::WebACLAssociation",
    "AWS::WAFRegional::XssMatchSet",
    "AWS::WAFv2::IPSet",
    "AWS::WAFv2::LoggingConfiguration",
    "AWS::WAFv2::RegexPatternSet",
    "AWS::WAFv2::RuleGroup",
    "AWS::WAFv2::WebACL",
    "AWS::WAFv2::WebACLAssociation",
    "AWS::Wisdom::Assistant",
    "AWS::Wisdom::AssistantAssociation",
    "AWS::Wisdom::KnowledgeBase",
    "AWS::WorkSpaces::ConnectionAlias",
    "AWS::WorkSpaces::Workspace",
    "AWS::WorkSpaces::WorkspacesPool",
    "AWS::WorkSpacesThinClient::Environment",
    "AWS::WorkSpacesWeb::BrowserSettings",
    "AWS::WorkSpacesWeb::IdentityProvider",
    "AWS::WorkSpacesWeb::IpAccessSettings",
    "AWS::WorkSpacesWeb::NetworkSettings",
    "AWS::WorkSpacesWeb::Portal",
    "AWS::WorkSpacesWeb::TrustStore",
    "AWS::WorkSpacesWeb::UserAccessLoggingSettings",
    "AWS::WorkSpacesWeb::UserSettings",
    "AWS::XRay::Group",
    "AWS::XRay::ResourcePolicy",
    "AWS::XRay::SamplingRule",
    "Alexa::ASK::Skill",
    "Module",
]
cached: list[str] = []



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import pytest

from cfnlint.rules.resources.iam.Permissions import Permissions


@pytest.fixture(scope="module")
def rule():
    rule = Permissions()
    yield rule


@pytest.mark.parametrize(
    "name,instance,err_count",
    [
        ("Valid string", "s3:GetObject", 0),
        ("Valid list", ["s3:GetObject"], 0),
        ("Invalid string", "s3:Foo", 1),
        ("Invalid list", ["s3:Foo"], 1),
        ("Valid astrisk for permission", ["s3:*"], 0),
        ("Valid all astrisk", ["*"], 0),
        ("Valid string with ending astrisk", "s3:Get*", 0),
        ("Valid string with starting astrisk", "s3:*Object", 0),
        ("Invalid list", ["s3:Foo"], 1),
        ("Invalid string with ending astrisk", "s3:Foo*", 1),
        ("Invalid string with starting astrisk", "s3:*Foo", 1),
        ("Invalid service", "foo:Bar", 1),
        ("Empty string", "", 1),
    ],
)
def test_permissions(name, instance, err_count, rule, validator):
    errors = list(rule.validate(validator, {}, instance, {}))
    assert len(errors) == err_count, f"Test {name!r} got {errors!r}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_rds_dbcluster
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class DbClusterServerlessExclusive(CfnLintJsonSchema):
    id = "E3686"
    shortdesc = "Validate allowed properties when using a serverless RDS DB cluster"
    description = (
        "Validate that when EngineMode is 'serverless' or 'provisioned' that the "
        "appropriate allowed properties are provided. If 'EngineMode' is not provided "
        "make sure serverless properties don't exist at all."
    )
    tags = ["resources"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::RDS::DBCluster/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_rds_dbcluster,
                filename="serverless_exclusive.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:

        # validator None means the schema is falsy
        if err.validator is None:
            if instance.get("EngineMode") in ["serverless", "provisioned"]:
                return (
                    f"EngineMode {instance.get('EngineMode')!r} "
                    f" doesn't allow additional properties {err.path[0]!r}"
                )
            else:
                return "Additional properties are not allowed " f"({err.path[0]!r})"

        return err.message



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from unittest import TestCase

from cfnlint.context import Context
from cfnlint.helpers import FUNCTIONS
from cfnlint.jsonschema import CfnTemplateValidator
from cfnlint.rules.resources.iam.IdentityPolicy import IdentityPolicy


class TestIdentityPolicies(TestCase):
    """Test IAM identity Policies"""

    def setUp(self):
        """Setup"""
        self.rule = IdentityPolicy()

    def test_object_basic(self):
        """Test Positive"""
        validator = CfnTemplateValidator()

        policy = {"Version": "2012-10-18"}

        errs = list(
            self.rule.validate(
                validator=validator, policy=policy, schema={}, policy_type=None
            )
        )
        self.assertEqual(errs[0].message, "'Statement' is a required property")
        self.assertListEqual(list(errs[0].path), [])
        self.assertEqual(len(errs), 2, errs)
        self.assertEqual(
            errs[1].message, "'2012-10-18' is not one of ['2008-10-17', '2012-10-17']"
        )
        self.assertListEqual(list(errs[1].path), ["Version"])

    def test_object_multiple_effect(self):
        validator = CfnTemplateValidator()

        policy = {
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Effect": "Allow",
                    "NotAction": "*",
                    "Action": [
                        "cloudformation:*",
                    ],
                    "Resource": "*",
                }
            ],
        }

        errs = list(
            self.rule.validate(
                validator=validator, policy=policy, schema={}, policy_type=None
            )
        )
        self.assertEqual(len(errs), 2, errs)
        self.assertEqual(
            errs[0].message,
            ("Only one of ['Action', 'NotAction'] is a required property"),
        )
        self.assertEqual(
            errs[1].message,
            ("Only one of ['Action', 'NotAction'] is a required property"),
        )
        self.assertIn(
            ["Statement", 0, "NotAction"], [list(errs[0].path), list(errs[1].path)]
        )
        self.assertIn(
            ["Statement", 0, "Action"], [list(errs[0].path), list(errs[1].path)]
        )

    def test_object_statements(self):
        validator = CfnTemplateValidator({}).evolve(
            context=Context(functions=FUNCTIONS)
        )

        policy = {
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Effect": "NotAllow",
                    "Action": [
                        "cloudformation:Describe*",
                        "cloudformation:List*",
                        "cloudformation:Get*",
                    ],
                    "Resource": [
                        {
                            "Fn::Sub": "arn:${AWS::Partition}:iam::123456789012:role/object-role"
                        },
                        {
                            "Fn::Sub": "arn:aws:cloudformation:${AWS::Region}:aws:transform/Serverless-2016-10-31"
                        },
                        {
                            "NotValid": [
                                "arn:${AWS::Partition}:iam::123456789012:role/object-role"
                            ]
                        },
                        "arn:aws:medialive:*",
                    ],
                }
            ],
        }

        errs = list(
            self.rule.validate(
                validator=validator, policy=policy, schema={}, policy_type=None
            )
        )
        self.assertEqual(len(errs), 2, errs)
        self.assertEqual(errs[0].message, "'NotAllow' is not one of ['Allow', 'Deny']")
        self.assertListEqual(list(errs[0].path), ["Statement", 0, "Effect"])
        self.assertEqual(
            errs[1].message,
            "{'NotValid': ['arn:${AWS::Partition}:iam::123456789012:role/object-role']} is not of type 'string'",
        )
        self.assertListEqual(list(errs[1].path), ["Statement", 0, "Resource", 2])

    def test_string_statements(self):
        """Test Positive"""
        validator = CfnTemplateValidator()

        # ruff: noqa: E501
        policy = """
            {
                "Version": "2012-10-18",
                "Statement": [
                    {
                        "Effect": "Allow",
                        "Action": [
                            "cloudformation:Describe*",
                            "cloudformation:List*",
                            "cloudformation:Get*"
                        ],
                        "Resource": [
                            "*",
                            {"Fn::Sub": ["arn:${AWS::Partition}:iam::123456789012/role/string-role"]}
                        ]
                    }
                ]
            }
        """

        errs = list(
            self.rule.validate(
                validator=validator, policy=policy, schema={}, policy_type=None
            )
        )
        self.assertEqual(len(errs), 2, errs)
        self.assertEqual(
            errs[0].message,
            "{'Fn::Sub': ['arn:${AWS::Partition}:iam::123456789012/role/string-role']} is not of type 'string'",
        )
        self.assertListEqual(list(errs[0].path), ["Statement", 0, "Resource", 1])
        self.assertEqual(
            errs[1].message, "'2012-10-18' is not one of ['2008-10-17', '2012-10-17']"
        )
        self.assertListEqual(list(errs[1].path), ["Version"])



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.functions.Base64 import Base64


@pytest.fixture(scope="module")
def rule():
    rule = Base64()
    yield rule


@pytest.mark.parametrize(
    "name,instance,schema,expected",
    [
        (
            "Valid Fn::Base64 string",
            {"Fn::Base64": "foo"},
            {"type": "string"},
            [],
        ),
        (
            "Invalid Fn::Base64 for wrong output type",
            {"Fn::Base64": "foo"},
            {"type": "array"},
            [
                ValidationError(
                    "{'Fn::Base64': 'foo'} is not of type 'array'",
                    path=deque([]),
                    schema_path=deque([]),
                    validator="fn_base64",
                ),
            ],
        ),
        (
            "Invalid Fn::Base64 is NOT a string",
            {"Fn::Base64": ["foo", "bar"]},
            {"type": "string"},
            [
                ValidationError(
                    "['foo', 'bar'] is not of type 'string'",
                    path=deque(["Fn::Base64"]),
                    schema_path=deque(["type"]),
                    validator="fn_base64",
                ),
            ],
        ),
        (
            "Invalid Fn::Base64 using an invalid function",
            {"Fn::Base64": {"foo": "bar"}},
            {"type": "string"},
            [
                ValidationError(
                    "{'foo': 'bar'} is not of type 'string'",
                    path=deque(["Fn::Base64"]),
                    schema_path=deque(["type"]),
                    validator="fn_base64",
                ),
            ],
        ),
        (
            "Valid Fn::Base64 with a valid function",
            {"Fn::Base64": {"Fn::Sub": "foo"}},
            {"type": "string"},
            [],
        ),
    ],
)
def test_validate(name, instance, schema, expected, rule, validator):
    errs = list(rule.fn_base64(validator, schema, instance, {}))
    assert errs == expected, f"Test {name!r} got {errs!r}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import CfnTemplateValidator, ValidationError
from cfnlint.rules.functions.Cidr import Cidr
from cfnlint.rules.functions.Join import Join
from cfnlint.rules.functions.Ref import Ref
from cfnlint.rules.functions.RefResolved import RefResolved
from cfnlint.rules.outputs.Value import Value  # pylint: disable=E0401


@pytest.fixture
def template():
    return {
        "Parameters": {
            "additionalVpcCidr": {
                "Type": "String",
                "Default": "",
            },
            "badAdditionalVpcCidr": {
                "Type": "String",
                "Default": "1",
            },
        },
        "Conditions": {
            "isAdditionalVpc": {
                "Fn::Not": [
                    {
                        "Fn::Equals": [
                            {"Ref": "additionalVpcCidr"},
                            "",
                        ]
                    }
                ]
            },
            "isBadAdditionalVpc": {
                "Fn::Not": [
                    {
                        "Fn::Equals": [
                            {"Ref": "badAdditionalVpcCidr"},
                            "",
                        ]
                    }
                ]
            },
        },
    }


@pytest.fixture
def validator(cfn):

    ref = Ref()
    ref.child_rules["W1030"] = RefResolved()
    yield CfnTemplateValidator(schema={}).extend(
        validators={
            "fn_join": Join().fn_join,
            "ref": ref.ref,
            "fn_cidr": Cidr().fn_cidr,
        }
    )(
        schema={},
        cfn=cfn,
    )


@pytest.mark.parametrize(
    "input,expected",
    [
        (
            {
                "Value": "foo",
            },
            [],
        ),
        (
            {
                "Value": 1.0,
            },
            [
                ValidationError(
                    "1.0 is not of type 'array', 'string'",
                    validator="type",
                    schema_path=deque(["type"]),
                    path=deque(["Value"]),
                    rule=Value(),
                )
            ],
        ),
        (
            {
                "Value": 1,
            },
            [
                ValidationError(
                    "1 is not of type 'array', 'string'",
                    validator="type",
                    schema_path=deque(["type"]),
                    path=deque(["Value"]),
                    rule=Value(),
                )
            ],
        ),
        (
            {"Value": True},
            [
                ValidationError(
                    "True is not of type 'array', 'string'",
                    validator="type",
                    schema_path=deque(["type"]),
                    path=deque(["Value"]),
                    rule=Value(),
                )
            ],
        ),
        (
            {"Value": [{}]},
            [
                ValidationError(
                    "{} is not of type 'string'",
                    validator="type",
                    schema_path=deque(["items", "type"]),
                    path=deque(["Value", 0]),
                    rule=Value(),
                )
            ],
        ),
        (
            {"Value": {"foo": "bar"}},
            [
                ValidationError(
                    "{'foo': 'bar'} is not of type 'array', 'string'",
                    validator="type",
                    schema_path=deque(["type"]),
                    path=deque(["Value"]),
                    rule=Value(),
                )
            ],
        ),
        (
            {
                "Condition": "isAdditionalVpc",
                "Value": {
                    "Fn::Join": [
                        ",",
                        {"Fn::Cidr": [{"Ref": "additionalVpcCidr"}, 3, 8]},
                    ]
                },
            },
            [],
        ),
        (
            {
                "Condition": "isBadAdditionalVpc",
                "Value": {
                    "Fn::Join": [
                        ",",
                        {"Fn::Cidr": [{"Ref": "badAdditionalVpcCidr"}, 3, 8]},
                    ]
                },
            },
            [
                ValidationError(
                    (
                        "{'Ref': 'badAdditionalVpcCidr'} does not match "
                        "'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\\\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\\\\/([0-9]|[1-2][0-9]|3[0-2]))$'"
                        " when 'Ref' is resolved"
                    ),
                    validator="ref",
                    schema_path=deque(
                        ["fn_join", "fn_items", "fn_cidr", "fn_items", "ref", "pattern"]
                    ),
                    path=deque(["Value", "Fn::Join", 1, "Fn::Cidr", 0, "Ref"]),
                    rule=RefResolved(),
                )
            ],
        ),
    ],
)
def test_output_value(input, expected, validator):
    rule = Value()
    results = list(rule.validate(validator, {}, input, {}))

    assert results == expected, f"Expected {expected} results, got {results}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.resources.ServerlessTransform import ServerlessTransform


@pytest.fixture(scope="module")
def rule():
    rule = ServerlessTransform()
    yield rule


@pytest.mark.parametrize(
    "name,instance,template,expected",
    [
        (
            "Valid with transform",
            "AWS::Serverless::Function",
            {
                "Transform": ["AWS::Serverless-2016-10-31"],
            },
            [],
        ),
        (
            "Invalid type",
            [],
            {},
            [],
        ),
        (
            "Valid type without transform",
            "AWS::Lambda::Function",
            {},
            [],
        ),
        (
            "Invalid when not specified",
            "AWS::Serverless::Function",
            {},
            [
                ValidationError(
                    (
                        "'AWS::Serverless::Function' type used "
                        "without the serverless transform "
                        "'AWS::Serverless-2016-10-31'"
                    ),
                    rule=ServerlessTransform(),
                )
            ],
        ),
    ],
    indirect=["template"],
)
def test_validate(name, instance, template, expected, rule, validator):
    errors = list(rule.validate(validator, False, instance, {}))
    assert errors == expected, f"Test {name!r} got {errors!r}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase

import cfnlint.decode.cfn_json  # pylint: disable=E0401
import cfnlint.decode.cfn_yaml  # pylint: disable=E0401
from cfnlint.config import _DEFAULT_RULESDIR
from cfnlint.rules import Rules


class TestDuplicate(BaseTestCase):
    """Test Duplicates Parsing"""

    def setUp(self):
        """SetUp template object"""
        self.rules = Rules()
        rulesdirs = [_DEFAULT_RULESDIR]
        for rulesdir in rulesdirs:
            self.rules.create_from_directory(rulesdir)

    def test_success_run(self):
        """Test success run"""

        filename = "test/fixtures/templates/good/generic.yaml"

        try:
            cfnlint.decode.cfn_yaml.load(filename)
        except cfnlint.decode.cfn_yaml.CfnParseError:
            assert False
            return

        assert True

    def test_fail_json_run(self):
        """Test failure run"""

    def test_fail_run(self):
        """Test failure run"""

        filename = "test/fixtures/templates/bad/duplicate.json"

        try:
            cfnlint.decode.cfn_json.load(filename)
        except cfnlint.decode.cfn_json.JSONDecodeError as e:
            self.assertEqual(len(e.matches), 3)
            return

        assert False

    def test_fail_yaml_run(self):
        """Test failure run"""

        filename = "test/fixtures/templates/bad/duplicate.yaml"

        try:
            cfnlint.decode.cfn_yaml.load(filename)
        except cfnlint.decode.cfn_yaml.CfnParseError as e:
            self.assertEqual(len(e.matches), 3)
            return

        assert False



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError

# ruff: noqa: E501
from cfnlint.rules.resources.elasticloadbalancingv2.LoadBalancerApplicationSubnets import (
    LoadBalancerApplicationSubnets,
)


@pytest.fixture(scope="module")
def rule():
    rule = LoadBalancerApplicationSubnets()
    yield rule


@pytest.mark.parametrize(
    "instance,expected",
    [
        (
            {"Type": "network"},
            [],
        ),
        (
            {"Type": "application", "Subnets": ["SubnetA", "SubnetB"]},
            [],
        ),
        (
            {"Type": "network", "Subnets": ["SubnetA"]},
            [],
        ),
        (
            {"Subnets": ["SubnetA"]},
            [
                ValidationError(
                    ("['SubnetA'] is too short (2)"),
                    rule=LoadBalancerApplicationSubnets(),
                    path=deque(["Subnets"]),
                    validator="minItems",
                    schema_path=deque(["then", "properties", "Subnets", "minItems"]),
                )
            ],
        ),
        (
            {
                "Type": "application",
                "Subnets": ["SubnetA"],
            },
            [
                ValidationError(
                    ("['SubnetA'] is too short (2)"),
                    rule=LoadBalancerApplicationSubnets(),
                    path=deque(["Subnets"]),
                    validator="minItems",
                    schema_path=deque(["then", "properties", "Subnets", "minItems"]),
                )
            ],
        ),
    ],
)
def test_backup_lifecycle(instance, expected, rule, validator):
    errs = list(rule.validate(validator, "", instance, {}))

    assert errs == expected, f"Expected {expected} got {errs}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.resources.rds.DbClusterMonitoring import DbClusterMonitoring


@pytest.fixture(scope="module")
def rule():
    rule = DbClusterMonitoring()
    yield rule


@pytest.mark.parametrize(
    "name,instance,expected",
    [
        (
            "Valid monitoring configuration",
            {
                "MonitoringInterval": 1,
                "MonitoringRoleArn": "arn",
            },
            [],
        ),
        (
            "Valid empty monitoring",
            {},
            [],
        ),
        (
            "Invalid with just monitoringInterval",
            {"MonitoringInterval": "1"},
            [
                ValidationError(
                    (
                        "You must have 'MonitoringRoleArn' specified "
                        "with 'MonitoringInterval' greater than 0"
                    ),
                    rule=DbClusterMonitoring(),
                    path=deque([]),
                    schema_path=deque(["else", "then", "required"]),
                    validator="required",
                )
            ],
        ),
        (
            "Invalid with just monitoringRoleArn",
            {
                "MonitoringRoleArn": "arn",
            },
            [
                ValidationError(
                    (
                        "You must have 'MonitoringRoleArn' specified "
                        "with 'MonitoringInterval' greater than 0"
                    ),
                    rule=DbClusterMonitoring(),
                    path=deque([]),
                    schema_path=deque(["then", "required"]),
                    validator="required",
                )
            ],
        ),
        (
            "Invalid with MonitoringInterval is 0",
            {
                "MonitoringRoleArn": "arn",
                "MonitoringInterval": "0",
            },
            [
                ValidationError(
                    (
                        "You must have 'MonitoringRoleArn' specified "
                        "with 'MonitoringInterval' greater than 0"
                    ),
                    rule=DbClusterMonitoring(),
                    path=deque(["MonitoringInterval"]),
                    schema_path=deque(
                        ["then", "properties", "MonitoringInterval", "exclusiveMinimum"]
                    ),
                    validator="exclusiveMinimum",
                )
            ],
        ),
    ],
)
def test_validate(name, instance, expected, rule, validator):
    errs = list(rule.validate(validator, "", instance, {}))
    assert errs == expected, f"Test {name!r} for expected {expected!r} got {errs!r}"



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any, Dict, Mapping, Sequence, Union

from sympy import And, Not, Or, Symbol
from sympy.logic.boolalg import BooleanFunction

from cfnlint.conditions._equals import Equal
from cfnlint.helpers import FUNCTION_CONDITIONS

# we leave the type hinting here
_CONDITION = Dict[str, Union[str, Sequence["_CONDITION"]]]


class Condition:
    """The generic class to represent any type of Condition"""

    def __init__(self) -> None:
        self._fn_equals: Equal | None = None
        self._condition: ConditionList | ConditionNamed | None = None

    def _init_condition(
        self,
        condition: _CONDITION,
        all_conditions: dict[str, dict],
    ) -> None:
        if len(condition) == 1:
            for k, v in condition.items():
                if k in FUNCTION_CONDITIONS:
                    if not isinstance(v, list):
                        raise ValueError(f"{k} value should be an array")
                    if k == "Fn::Equals":
                        self._fn_equals = Equal(v)
                    elif k == "Fn::And":
                        self._condition = ConditionAnd(v, all_conditions)
                    elif k == "Fn::Or":
                        self._condition = ConditionOr(v, all_conditions)
                    elif k == "Fn::Not":
                        self._condition = ConditionNot(v, all_conditions)
                elif k == "Condition":
                    if not isinstance(v, str):
                        raise ValueError(f"Condition value {v!r} must be a string")
                    self._condition = ConditionNamed(v, all_conditions)
                else:
                    raise ValueError(f"Unknown key ({k}) in condition")
        else:
            raise ValueError("Condition value must be an object of length 1")

    @property
    def equals(self) -> list[Equal]:
        """Returns a Sequence of the Equals that make up the Condition

        Args: None

        Returns:
            Sequence[EqualParameter] | Sequence[Equal] | None:
                The Equal that are part of the condition
        """
        if self._fn_equals:
            return [self._fn_equals]
        if self._condition:
            return self._condition.equals
        return []

    def build_cnf(self, params: dict[str, Symbol]) -> BooleanFunction | Symbol | None:
        """Build a SymPy CNF based on the provided params

        Args:
            params dict[str, Symbol]: params is a dict that represents
                    the hash of an Equal and the SymPy Symbols

        Returns:
            Any: Any number of different SymPy CNF clauses
        """
        if self._condition:
            return self._condition.build_cnf(params)
        if self._fn_equals:
            return params.get(self._fn_equals.hash)
        return None

    def _test(self, scenarios: Mapping[str, str]) -> bool:
        if self._fn_equals:
            return self._fn_equals.test(scenarios)
        if self._condition:
            # pylint: disable=W0212
            return self._condition._test(scenarios)
        return False


class ConditionList(Condition):
    """The generic class to represent any type of List condition
    List conditions are And, Or, Not
    """

    def __init__(
        self, conditions: Sequence[_CONDITION], all_conditions: dict[str, dict]
    ) -> None:
        super().__init__()
        self._conditions: list[ConditionUnnammed] = []
        self._prefix_path: str = ""
        for condition in conditions:
            self._conditions.append(ConditionUnnammed(condition, all_conditions))

    @property
    def equals(self) -> list[Equal]:
        """Returns a List of the Equals that make up the Condition

        Args: None

        Returns:
            list[Equal]: The Equal that are part of the condition
        """
        equals: list[Equal] = []
        for condition in self._conditions:
            equals.extend(condition.equals)
        return equals


class ConditionAnd(ConditionList):
    """Represents the logic specific to an And Condition"""

    def __init__(
        self, conditions: Sequence[_CONDITION], all_conditions: dict[str, dict]
    ) -> None:
        super().__init__(conditions, all_conditions)
        self._prefix_path = "Fn::And"

    def build_cnf(self, params: dict[str, Symbol]) -> BooleanFunction:
        """Build a SymPy CNF solver based on the provided params
        Args:
            params dict[str, Symbol]: params is a dict that represents
                    the hash of an Equal and the SymPy Symbols
        Returns:
            BooleanFunction: An And SymPy BooleanFunction
        """
        conditions: list[Any] = []
        for child in self._conditions:
            conditions.append(child.build_cnf(params))

        return And(*conditions)

    def _test(self, scenarios: Mapping[str, str]) -> bool:
        # pylint: disable=W0212
        return all(condition._test(scenarios) for condition in self._conditions)


class ConditionNot(ConditionList):
    """Represents the logic specific to an Not Condition"""

    def __init__(
        self, conditions: Sequence[_CONDITION], all_conditions: dict[str, dict]
    ) -> None:
        super().__init__(conditions, all_conditions)
        self._prefix_path = "Fn::Not"
        if len(conditions) != 1:
            raise ValueError("Condition length must be 1")

    def build_cnf(self, params: dict[str, Symbol]) -> BooleanFunction:
        """Build a SymPy CNF solver based on the provided params
        Args:
            params dict[str, Symbol]: params is a dict that represents
                    the hash of an Equal and the SymPy Symbol
        Returns:
            BooleanFunction: A Not SymPy BooleanFunction
        """
        return Not(self._conditions[0].build_cnf(params))

    def _test(self, scenarios: Mapping[str, str]) -> bool:
        # pylint: disable=W0212
        return not any(condition._test(scenarios) for condition in self._conditions)


class ConditionOr(ConditionList):
    """Represents the logic specific to an Or Condition"""

    def __init__(
        self, conditions: Sequence[_CONDITION], all_conditions: dict[str, dict]
    ) -> None:
        super().__init__(conditions, all_conditions)
        self._prefix_path = "Fn::Or"

    def build_cnf(self, params: dict[str, Symbol]) -> BooleanFunction:
        """Build a SymPy CNF solver based on the provided params
        Args:
            params dict[str, Symbol]: params is a dict that represents
                    the hash of an Equal and the SymPy Symbols
        Returns:
            BooleanFunction: An Or SymPy BooleanFunction
        """
        conditions: list[Any] = []
        for child in self._conditions:
            conditions.append(child.build_cnf(params))
        return Or(*conditions)

    def _test(self, scenarios: Mapping[str, str]) -> bool:
        # pylint: disable=W0212
        return any(condition._test(scenarios) for condition in self._conditions)


class ConditionUnnammed(Condition):
    """Represents an unnamed condition which is basically a nested Equals"""

    def __init__(self, condition: Any, all_conditions: dict[str, dict]) -> None:
        super().__init__()
        if isinstance(condition, dict):
            self._init_condition(condition, all_conditions)
        else:
            raise ValueError("Condition must have a value that is an object")


class ConditionNamed(Condition):
    """The parent condition that directly represents a named condition in a template"""

    def __init__(self, name: str, all_conditions: dict[str, dict]) -> None:
        super().__init__()
        condition = all_conditions.get(name)
        if isinstance(condition, dict):
            self._name = name
            self._init_condition(condition, all_conditions)
        else:
            raise ValueError(f"Condition {name} must have a value that is an object")

    def build_true_cnf(self, params: dict[str, Symbol]) -> Any:
        """Build a SymPy CNF for a True based scenario
        Args:
            params dict[str, Symbol]: params is a dict that represents
                    the hash of an Equal and the SymPy Symbols
        Returns:
            Any: A SymPy CNF clause
        """
        return self.build_cnf(params)

    def build_false_cnf(self, params: dict[str, Symbol]) -> Any:
        """Build a SymPy CNF for a False based scenario
        Args:
            params dict[str, Symbol]: params is a dict that represents
                    the hash of an Equal and the SymPy CNF Symbols
        Returns:
            Any: A Not SymPy CNF clause
        """
        return Not(self.build_true_cnf(params))

    def test(self, scenarios: Mapping[str, str]) -> bool:
        """Test a condition based on a scenario"""
        return self._test(scenarios)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase

from cfnlint.helpers import RegexDict


class TestRegexDict(BaseTestCase):
    """Test Regex Dict"""

    def test_getitem(self):
        """Test success run"""

        obj = RegexDict()
        obj["^Value$"] = True

        with self.assertRaises(KeyError):
            obj["NotExist"]

    def test_get(self):
        obj = RegexDict()
        obj["^Value$"] = True

        self.assertEqual(obj.get("NotExist", "Default"), "Default")

    def test_return_longest(self):
        obj = RegexDict()
        obj["^Test"] = False
        obj["^TestLonger.*"] = True

        self.assertTrue(obj["TestLongerObject"])

    def test_contains_object(self):
        obj = RegexDict()
        obj["^Test"] = False
        obj["^TestLonger.*"] = True

        self.assertTrue("TestLongerObject" in obj)
        self.assertFalse("NotIn" in obj)
        self.assertFalse({"a": "b"} in obj)



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from dataclasses import dataclass
from typing import Dict


@dataclass
class SchemaPatch:
    included_resource_types: list[str]
    excluded_resource_types: list[str]
    patches: dict[str, list[Dict]]

    @staticmethod
    def from_dict(value: Dict):
        return SchemaPatch(
            included_resource_types=value.get("IncludeResourceTypes", []),
            excluded_resource_types=value.get("ExcludeResourceTypes", []),
            patches=value.get("Patches", {}),
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.helpers import valid_snapshot_types
from cfnlint.jsonschema import Validator
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema


class UpdateReplacePolicy(CfnLintJsonSchema):
    """Check Base Resource Configuration"""

    id = "E3036"
    shortdesc = "Check UpdateReplacePolicy values for Resources"
    description = "Check that the UpdateReplacePolicy values are valid"
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-attribute-updatereplacepolicy.html"
    tags = ["resources", "updatereplacepolicy"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/*/UpdateReplacePolicy"],
            all_matches=True,
        )

    def validate(self, validator: Validator, dP: str, instance, schema):
        enum = ["Delete", "Retain"]
        resource_name = validator.context.path.path[1]
        if (
            isinstance(resource_name, str)
            and validator.context.resources[resource_name].type in valid_snapshot_types
        ):
            enum.append("Snapshot")

        validator = validator.evolve(
            context=validator.context.evolve(
                functions=[
                    "Fn::Sub",
                    "Fn::Select",
                    "Fn::FindInMap",
                    "Fn::If",
                    "Ref",
                ]
            ),
            schema={"type": "string", "enum": enum},
        )

        yield from self._iter_errors(validator, instance)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_ec2_instance
import cfnlint.data.schemas.extensions.aws_ec2_securitygroup
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class Ebs(CfnLintJsonSchema):
    id = "E3671"
    shortdesc = "Validate block device mapping configuration"
    description = "Certain volume types require Iops to be specified"
    tags = ["resources", "ec2"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::AutoScaling::LaunchConfiguration/Properties/BlockDeviceMappings/*/Ebs",
                "Resources/AWS::EC2::Instance/Properties/BlockDeviceMappings/*/Ebs",
                "Resources/AWS::EC2::LaunchTemplate/Properties/LaunchTemplateData/BlockDeviceMappings/*/Ebs",
                "Resources/AWS::EC2::SpotFleet/Properties/SpotFleetRequestConfigData/LaunchSpecifications/*/BlockDeviceMappings/*/Ebs",
                "Resources/AWS::OpsWorks::Instance/Properties/BlockDeviceMappings/*/Ebs",
            ],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_ec2_instance,
                filename="blockdevicemappings.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        if err.schema_path[0] == "allOf":
            if err.schema_path[1] == 1:
                return (
                    "Additional properties are not allowed (Iops) "
                    "was unexpected when 'VolumeType' has a value "
                    f"of {instance.get('VolumeType')!r}"
                )

        return (
            f"'Iops' is a required property when 'VolumeType' has a value "
            f"of {instance.get('VolumeType')!r}"
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_rds_dbcluster
from cfnlint.jsonschema import ValidationResult, Validator
from cfnlint.rules.jsonschema.CfnLintJsonSchema import SchemaDetails
from cfnlint.rules.jsonschema.CfnLintJsonSchemaRegional import CfnLintJsonSchemaRegional


class DbClusterInstanceClassEnum(CfnLintJsonSchemaRegional):
    id = "E3694"
    shortdesc = "Validates RDS DB Cluster instance class"
    description = (
        "Validates the RDS DB Cluster instance types based on region "
        "and data gathered from the pricing APIs"
    )
    tags = ["resources"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::RDS::DBCluster/Properties"],
            schema_details=SchemaDetails(
                cfnlint.data.schemas.extensions.aws_rds_dbcluster,
                "dbclusterinstanceclass_enum.json",
            ),
        )

    def validate(
        self, validator: Validator, keywords: Any, instance: Any, schema: dict[str, Any]
    ) -> ValidationResult:
        # RDS pricing schemas are based on values
        validator = validator.evolve(
            context=validator.context.evolve(
                functions=[],
            )
        )

        yield from super().validate(validator, keywords, instance, schema)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.transforms.Configuration import Configuration


@pytest.fixture(scope="module")
def rule():
    rule = Configuration()
    yield rule


@pytest.mark.parametrize(
    "name,instance,expected",
    [
        (
            "Empty list is ok",
            [],
            [],
        ),
        (
            "String is ok",
            "Foo",
            [],
        ),
        (
            "List is ok",
            ["Foo", "Bar"],
            [],
        ),
        (
            "Object is ok",
            {"Name": "Foo", "Parameters": {"Bar": "Test"}},
            [],
        ),
        (
            "Array of objects is ok",
            [
                {"Name": "Foo", "Parameters": {"Bar": "Test"}},
                "Foo",
            ],
            [],
        ),
        (
            "Missing required Name",
            {"Parameters": {"Bar": "Test"}},
            [
                ValidationError(
                    "'Name' is a required property",
                    validator="required",
                    rule=Configuration(),
                    path=deque([]),
                    schema_path=deque(["required"]),
                )
            ],
        ),
        (
            "No additional property names are allowed",
            {"Name": "Foo", "Foo": "Bar", "Parameters": {"Bar": "Test"}},
            [
                ValidationError(
                    "Additional properties are not allowed ('Foo' was unexpected)",
                    validator="additionalProperties",
                    rule=Configuration(),
                    path=deque(["Foo"]),
                    schema_path=deque(["additionalProperties"]),
                )
            ],
        ),
        (
            "Null is not ok",
            None,
            [
                ValidationError(
                    "None is not of type 'string', 'array', 'object'",
                    validator="type",
                    rule=Configuration(),
                    path=deque([]),
                    schema_path=deque(["type"]),
                )
            ],
        ),
    ],
)
def test_validate(name, instance, expected, rule, validator):
    errs = list(rule.validate(validator, {}, instance, {}))

    assert errs == expected, f"Test {name!r} got {errs!r}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.outputs.ImportValue import ImportValue


@pytest.fixture(scope="module")
def rule():
    rule = ImportValue()
    yield rule


@pytest.mark.parametrize(
    "name,instance,path,schema,expected",
    [
        (
            "Valid Fn::ImportValue",
            {"Fn::ImportValue": "Export"},
            {
                "path": ["Resources", "MyResource", "Properties", "BucketName"],
            },
            {"type": "string"},
            [],
        ),
        (
            "Short path",
            {"Fn::ImportValue": "Export"},
            {
                "path": ["Outputs"],
            },
            {"type": "object"},
            [],
        ),
        (
            "Invalid Fn::ImportValue in Output",
            {"Fn::ImportValue": "Export"},
            {
                "path": ["Outputs", "MyOutput", "Value"],
            },
            {"type": "string"},
            [
                ValidationError(
                    (
                        "The output value {'Fn::ImportValue': 'Export'} "
                        "is an import from another output"
                    ),
                    path=deque(["Fn::ImportValue"]),
                    rule=ImportValue(),
                )
            ],
        ),
    ],
    indirect=["path"],
)
def test_validate(name, instance, path, schema, expected, rule, validator):
    errs = list(rule.validate(validator, schema, instance, {}))
    assert errs == expected, f"Test {name!r} got {errs!r}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import logging

from cfnlint._typing import RuleMatches
from cfnlint.rules import CloudFormationLintRule, RuleMatch
from cfnlint.template import Template

LOGGER = logging.getLogger("cfnlint")


class ForEach(CloudFormationLintRule):
    id = "E1032"
    shortdesc = "Validates ForEach functions"
    description = "Validates that ForEach parameters have a valid configuration"
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-getatt.html"
    tags = ["functions", "foreach"]

    def validate_transform_is_declared(
        self, has_language_extensions_transform, tree, intrinsic_function
    ):
        matches = []
        if not has_language_extensions_transform:
            message = (
                "Missing Transform: Declare the AWS::LanguageExtensions Transform"
                " globally to enable use of the intrinsic function "
                + intrinsic_function
                + " at {0}"
            )
            matches.append(RuleMatch(tree[:], message.format("/".join(map(str, tree)))))
        return matches

    # pylint: disable=unused-argument
    def match(self, cfn: Template) -> RuleMatches:
        matches = []
        intrinsic_function = "Fn::ForEach"
        for_eaches = cfn.transform_pre["Fn::ForEach"]

        for for_each in for_eaches:
            has_language_extensions_transform = cfn.has_language_extensions_transform()

            matches.extend(
                self.validate_transform_is_declared(
                    has_language_extensions_transform,
                    for_each[:-1],
                    intrinsic_function,
                )
            )

        return matches



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.context import Context
from cfnlint.context.context import Path
from cfnlint.jsonschema import CfnTemplateValidator
from cfnlint.jsonschema._filter import FunctionFilter


@pytest.fixture(scope="module")
def filter():
    filter = FunctionFilter()
    yield filter


@pytest.mark.parametrize(
    "name,instance,schema,path,functions,expected",
    [
        (
            "Don't validate dynamic references inside of function",
            "{{resolve:ssm:${AWS::AccountId}/${AWS::Region}/ac}}",
            {"enum": "Foo"},
            deque(["Foo", "Test", "Fn::Sub"]),
            [],
            [],
        ),
        (
            "Validate dynamic references",
            "{{resolve:ssm:secret}}",
            {"enum": "Foo"},
            deque(["Foo", "Test"]),
            [],
            [
                ("{{resolve:ssm:secret}}", {"dynamicReference": {"enum": "Foo"}}),
            ],
        ),
        (
            "Lack of functions returns the schema and instance",
            {
                "Foo": {"Ref": "AWS::NoValue"},
            },
            {"required": ["Foo"]},
            deque([]),
            [],
            [
                (
                    {
                        "Foo": {"Ref": "AWS::NoValue"},
                    },
                    {"required": ["Foo"]},
                ),
            ],
        ),
        (
            "Filtered schemas",
            {
                "Foo": {"Ref": "AWS::NoValue"},
            },
            {"required": ["Foo"]},
            deque([]),
            ["Ref", "Fn::If"],
            [
                (
                    {},
                    {"required": ["Foo"]},
                ),
                ({"Foo": {"Ref": "AWS::NoValue"}}, {"cfnLint": [""]}),
            ],
        ),
    ],
)
def test_filter(name, instance, schema, path, functions, expected, filter):
    validator = CfnTemplateValidator(
        context=Context(
            regions=["us-east-1"],
            path=Path(path),
            functions=functions,
        ),
        schema=schema,
    )
    results = list(filter.filter(validator, instance, schema))

    assert results == expected, f"For test {name} got {results!r}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.jsonschema import CfnTemplateValidator, ValidationError
from cfnlint.rules import CloudFormationLintRule
from cfnlint.rules.jsonschema.MaxProperties import MaxProperties


class ChildMaxProperties(CloudFormationLintRule):
    id = "ChildMaxProperties"

    def foo(self, validator, awsType, instance, schema):
        if instance == "bar":
            return
        yield ValidationError("Not bar")


class TestMaxProperties(BaseRuleTestCase):
    """Test AWS Types"""

    def setUp(self):
        """Setup"""
        super(TestMaxProperties, self).setUp()
        self.rule = MaxProperties("ChildMaxProperties")
        self.rule.child_rules = {"ChildMaxProperties": ChildMaxProperties()}

    def test_max_properties(self):
        validator = CfnTemplateValidator({})
        self.assertEqual(
            len(list(self.rule.maxProperties(validator, 5, {"foo": True}, {}))),
            0,
        )

        errs = list(
            self.rule.maxProperties(validator, 1, {"foo": True, "bar": True}, {})
        )
        self.assertEqual(len(errs), 1)
        self.assertIsNone(errs[0].rule)

        errs = list(self.rule.maxProperties(validator, 1, {"foo": True}, {}))
        self.assertEqual(len(errs), 1)
        self.assertEqual(errs[0].rule.id, "ChildMaxProperties")

        errs = list(self.rule.maxProperties(validator, 1, [], {}))
        self.assertListEqual(errs, [])


class ChildMaxPropertiesWithFunction(CloudFormationLintRule):
    id = "ChildMaxPropertiesWithFunction"

    def maxProperties(self, validator, s, instance, schema):
        yield ValidationError("test", rule=self)
        yield ValidationError("test", rule=self)


class TestMaxPropertiesWithFunction(BaseRuleTestCase):
    """Test AWS Types"""

    def setUp(self):
        """Setup"""
        super(TestMaxPropertiesWithFunction, self).setUp()
        self.rule = MaxProperties("ChildMaxPropertiesWithFunction")
        self.rule.child_rules = {
            "ChildMaxPropertiesWithFunction": ChildMaxPropertiesWithFunction()
        }

    def test_property_names(self):
        validator = CfnTemplateValidator({})

        errs = list(self.rule.maxProperties(validator, 1, {"foo": True}, {}))
        self.assertEqual(len(errs), 2)
        self.assertEqual(errs[0].rule.id, "ChildMaxPropertiesWithFunction")


class TestMaxPropertiesNoChild(BaseRuleTestCase):
    """Test AWS Types"""

    def setUp(self):
        """Setup"""
        super(TestMaxPropertiesNoChild, self).setUp()
        self.rule = MaxProperties("ChildMaxProperties")
        self.rule.child_rules = {"ChildMaxProperties": None}

    def test_property_names(self):
        validator = CfnTemplateValidator({})

        errs = list(self.rule.maxProperties(validator, 1, {"foo": True}, {}))
        self.assertEqual(len(errs), 0)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules import CloudFormationLintRule


class DependentExcluded(CloudFormationLintRule):
    """Check Required Resource Configuration"""

    id = "E3020"
    shortdesc = (
        "Validate that when a property is specified another property should be excluded"
    )
    description = (
        "When certain properties are specified other properties should not be included"
    )
    source_url = "https://github.com/aws-cloudformation/cfn-lint/blob/main/docs/cfn-schema-specification.md#dependentexcluded"
    tags = ["resources"]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

from cfnlint.helpers import FUNCTIONS
from cfnlint.jsonschema import Validator
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema


class Export(CfnLintJsonSchema):
    """Check if Output Export values"""

    id = "E6102"
    shortdesc = "Validate that output exports have values of strings"
    description = "Make sure output exports have a value of type string"
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/outputs-section-structure.html"
    tags = ["outputs"]

    def __init__(self):
        super().__init__(
            keywords=["Outputs/*/Export/Name"],
            all_matches=True,
        )

    def validate(self, validator: Validator, _: Any, instance: Any, schema: Any):
        validator = validator.evolve(
            context=validator.context.evolve(
                resources={},
                functions=list(FUNCTIONS),
            ),
            schema={"type": "string"},
        )

        yield from self._iter_errors(validator, instance)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_rds_dbcluster
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class DbClusterSourceDBClusterIdentifier(CfnLintJsonSchema):
    id = "W3689"
    shortdesc = "When using a source DB certain properties are ignored"
    description = (
        "When creating a DBCluster from a source certain properties "
        "are ignored and could result in drift"
    )
    tags = ["resources", "rds"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::RDS::DBCluster/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_rds_dbcluster,
                filename="sourcedbclusteridentifier.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return (
            "Additional properties are ignored ('MasterUsername', "
            "'MasterUserPassword', 'StorageEncrypted')"
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

from cfnlint.jsonschema import ValidationResult, Validator
from cfnlint.rules.functions._BaseFn import BaseFn, all_types


class If(BaseFn):
    """Check if Condition exists"""

    id = "E1028"
    shortdesc = "Check Fn::If structure for validity"
    description = "Check Fn::If to make sure its valid.  Condition has to be a string."
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-conditions.html#intrinsic-function-reference-conditions-if"
    tags = ["functions", "if"]

    def __init__(self) -> None:
        super().__init__("Fn::If", all_types)

    def schema(self, validator, instance) -> dict[str, Any]:
        return {
            "type": ["array"],
            "minItems": 3,
            "maxItems": 3,
            "fn_items": [
                {
                    "functions": [],
                    "schema": {
                        "type": ["string"],
                        "enum": list(validator.context.conditions.conditions.keys()),
                    },
                },
            ],
        }

    def fn_if(
        self, validator: Validator, s: Any, instance: Any, schema: Any
    ) -> ValidationResult:
        # validate this function will return the correct type
        errs = list(self.validate_fn_output_types(validator, s, instance))

        key, value = self.key_value(instance)

        errs.extend(
            list(
                self.fix_errors(
                    self.validator(validator).descend(
                        value,
                        self.schema(validator, instance),
                        path=key,
                    )
                )
            )
        )

        if errs:
            yield from iter(errs)
            return

        for i in [1, 2]:
            # we pass through the functions for the paths down
            # the second and third element of the if
            try:
                element_validator = validator.evolve(
                    context=validator.context.evolve(
                        path=validator.context.path.descend(
                            path=key,
                        ),
                        conditions=validator.context.conditions.evolve(
                            {value[0]: True if i == 1 else False}
                        ),
                    )
                )
                for err in element_validator.descend(
                    instance=value[i], schema=s, path=i
                ):
                    err.path.appendleft(key)
                    yield err
            except ValueError:
                # impossible code path
                pass



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from collections import deque
from typing import Any

import cfnlint.data.schemas.extensions.aws_ecs_taskdefinition
from cfnlint.jsonschema import ValidationResult
from cfnlint.jsonschema.protocols import Validator
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class TaskFargateProperties(CfnLintJsonSchema):
    id = "E3048"
    shortdesc = "Validate ECS Fargate tasks have required properties and values"
    description = (
        "When using a ECS Fargate task there is a specfic combination "
        "of required properties and values"
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html#cfn-ecs-taskdefinition-memory"
    tags = ["properties", "ecs", "service", "container", "fargate"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::ECS::TaskDefinition/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_ecs_taskdefinition,
                filename="fargate_properties.json",
            ),
            all_matches=True,
        )

    def validate(
        self, validator: Validator, keywords: Any, instance: Any, schema: dict[str, Any]
    ) -> ValidationResult:
        for err in super().validate(validator, keywords, instance, schema):
            if err.validator == "not":
                err.message = "'PlacementConstraints' isn't supported for Fargate tasks"
                err.path = deque(["PlacementConstraints"])
                yield err
                continue
            yield err



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.resources.rds.DbClusterAurora import DbClusterAurora


@pytest.fixture(scope="module")
def rule():
    rule = DbClusterAurora()
    yield rule


@pytest.mark.parametrize(
    "instance,expected",
    [
        (
            {
                "Engine": "aurora-mysql",
            },
            [],
        ),
        (
            {"Engine": "aurora-mysql", "StorageType": "aurora"},
            [],
        ),
        (
            {"Engine": "aurora-mysql", "StorageType": "io1"},
            [
                ValidationError(
                    ("'io1' is not one of ['aurora', 'aurora-iopt1']"),
                    rule=DbClusterAurora(),
                    path=deque(["StorageType"]),
                    validator="enum",
                    schema_path=deque(
                        ["then", "properties", "StorageType", "then", "enum"]
                    ),
                ),
            ],
        ),
        (
            {
                "Engine": "aurora-mysql",
                "AllocatedStorage": 100,
                "DBClusterInstanceClass": "foo",
                "Iops": 100,
            },
            [
                ValidationError(
                    (
                        "Additional properties are not allowed "
                        "'AllocatedStorage' when creating Aurora cluster"
                    ),
                    rule=DbClusterAurora(),
                    path=deque(["AllocatedStorage"]),
                    validator=None,
                    schema_path=deque(["then", "properties", "AllocatedStorage"]),
                ),
                ValidationError(
                    (
                        "Additional properties are not allowed "
                        "'DBClusterInstanceClass' when creating Aurora cluster"
                    ),
                    rule=DbClusterAurora(),
                    path=deque(["DBClusterInstanceClass"]),
                    schema_path=deque(["then", "properties", "DBClusterInstanceClass"]),
                    validator=None,
                ),
                ValidationError(
                    (
                        "Additional properties are not allowed 'Iops' "
                        "when creating Aurora cluster"
                    ),
                    rule=DbClusterAurora(),
                    path=deque(["Iops"]),
                    schema_path=deque(["then", "properties", "Iops"]),
                    validator=None,
                ),
            ],
        ),
    ],
)
def test_validate(instance, expected, rule, validator):
    errs = list(rule.validate(validator, "", instance, {}))
    assert errs == expected, f"Expected {expected} got {errs}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import logging
from test.testlib.testcase import BaseTestCase

import cfnlint.config  # pylint: disable=E0401

LOGGER = logging.getLogger("cfnlint")


class TestTempalteArgs(BaseTestCase):
    """Test ConfigParser Arguments"""

    def tearDown(self):
        """Setup"""
        for handler in LOGGER.handlers:
            LOGGER.removeHandler(handler)

    def test_template_args(self):
        """test template args"""
        config = cfnlint.config.TemplateArgs(
            {
                "Metadata": {
                    "cfn-lint": {
                        "config": {
                            "regions": ["us-east-1", "us-east-2"],
                            "ignore_checks": ["E2530"],
                            "configure_rules": {"E3012": {"strict": "false"}},
                        }
                    }
                }
            }
        )

        self.assertEqual(config.template_args["regions"], ["us-east-1", "us-east-2"])
        self.assertEqual(
            config.template_args["configure_rules"], {"E3012": {"strict": "false"}}
        )

    def test_template_args_failure_bad_format(self):
        """test template args"""
        config = cfnlint.config.TemplateArgs(
            {
                "Metadata": {
                    "cfn-lint": {
                        "config": {"configure_rules": [{"E3012": {"strict": "false"}}]}
                    }
                }
            }
        )

        self.assertEqual(config.template_args.get("configure_rules"), None)

    def test_template_args_failure_bad_value(self):
        """test template args"""
        config = cfnlint.config.TemplateArgs(
            {
                "Metadata": {
                    "cfn-lint": {
                        "config": {
                            "configure_rules": [{"E3012": {"bad_value": "false"}}]
                        }
                    }
                }
            }
        )

        self.assertEqual(config.template_args.get("configure_rules"), None)

    def test_template_args_failure_good_and_bad_value(self):
        """test template args"""
        config = cfnlint.config.TemplateArgs(
            {
                "Metadata": {
                    "cfn-lint": {
                        "config": {
                            "configure_rules": [
                                {
                                    "A1": {"strict": "false"},
                                    "E3012": {"strict": "false"},
                                    "Z1": {"strict": "false"},
                                }
                            ]
                        }
                    }
                }
            }
        )

        self.assertEqual(config.template_args.get("configure_rules"), None)

    def test_bad_template_structure(self):
        """test template args"""
        config = cfnlint.config.TemplateArgs([])

        self.assertEqual(config._template_args, {})

    def test_bad_config_structure(self):
        """test template args"""
        config = cfnlint.config.TemplateArgs({"Metadata": {"cfn-lint": {"config": []}}})

        self.assertEqual(config._template_args, {})



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase

import cfnlint.decode.cfn_yaml  # pylint: disable=E0401
from cfnlint import ConfigMixIn
from cfnlint.config import _DEFAULT_RULESDIR
from cfnlint.rules import Rules
from cfnlint.runner import TemplateRunner


class TestCustomRuleParsing(BaseTestCase):
    """Test Node Objects"""

    def setUp(self):
        """SetUp template object"""
        self.rules = Rules()
        rulesdirs = [_DEFAULT_RULESDIR]
        for rulesdir in rulesdirs:
            self.rules.create_from_directory(rulesdir)

        self.filenames = {
            "generic_good": {"filename": "test/fixtures/templates/good/generic.yaml"}
        }

        self.perfect_rule = "test/fixtures/custom_rules/good/custom_rule_perfect.txt"
        self.valid_boolean = "test/fixtures/custom_rules/good/custom_rule_boolean.txt"
        self.invalid_boolean = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_boolean.txt"
        )
        self.invalid_op = "test/fixtures/custom_rules/bad/custom_rule_invalid_op.txt"
        self.invalid_prop = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_prop.txt"
        )
        self.invalid_propkey = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_propkey.txt"
        )
        self.invalid_rt = "test/fixtures/custom_rules/bad/custom_rule_invalid_rt.txt"
        self.invalid_equal = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_equal.txt"
        )
        self.invalid_not_equal = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_not_equal.txt"
        )
        self.invalid_set = "test/fixtures/custom_rules/bad/custom_rule_invalid_set.txt"
        self.invalid_not_set = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_not_set.txt"
        )
        self.invalid_greater_than = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_greater_than.txt"
        )
        self.invalid_less_than = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_less_than.txt"
        )
        self.invalid_regex = (
            "test/fixtures/custom_rules/bad/custom_rule_invalid_regex.txt"
        )

    def test_perfect_parse(self):
        """Test Successful Custom_Rule Parsing"""
        assert self.run_tests(self.perfect_rule) == []

    def test_invalid_op(self):
        """Test Successful Custom_Rule Parsing"""
        assert self.run_tests(self.invalid_op)[0].message.find("not in supported") > -1

    def test_invalid_set(self):
        """Test Successful Custom_Rule Parsing"""
        assert self.run_tests(self.invalid_set)[0].message.find("In set check") > -1

    def test_invalid_not_set(self):
        """Test Successful Custom_Rule Parsing"""
        assert self.run_tests(self.invalid_not_set)[0].message.find("Not in set") > -1

    def test_invalid_equal(self):
        """Test Successful Custom_Rule Parsing"""
        assert (
            self.run_tests(self.invalid_equal)[0].message.find("Must equal check") > -1
        )

    def test_invalid_not_equal(self):
        """Test Successful Custom_Rule Parsing"""
        assert (
            self.run_tests(self.invalid_not_equal)[0].message.find("Must not equal")
            > -1
        )

    def test_invalid_greater_than(self):
        """Test Successful Custom_Rule Parsing"""
        assert (
            self.run_tests(self.invalid_greater_than)[0].message.find(
                "Greater than check"
            )
            > -1
        )

    def test_invalid_less_than(self):
        """Test Successful Custom_Rule Parsing"""
        assert (
            self.run_tests(self.invalid_less_than)[0].message.find("Lesser than check")
            > -1
        )

    def test_invalid_regex(self):
        """Test Successful Custom_Rule Parsing"""
        assert (
            self.run_tests(self.invalid_regex)[0].message.find("Regex does not match")
            > -1
        )

    def test_valid_boolean_value(self):
        """Test Boolean values"""
        assert self.run_tests(self.valid_boolean) == []

    def test_invalid_boolean_value(self):
        """Test Boolean values"""
        assert (
            self.run_tests(self.invalid_boolean)[0].message.find(
                "Must equal check failed"
            )
            > -1
        )

    def test_invalid_prop(self):
        """Test Successful Custom_Rule Parsing"""
        assert self.run_tests(self.invalid_prop) == []

    def test_invalid_propKey(self):
        """Test Successful Custom_Rule Parsing"""
        assert self.run_tests(self.invalid_propkey) == []

    def test_invalid_resource_type(self):
        """Test Successful Custom_Rule Parsing"""
        assert self.run_tests(self.invalid_rt) == []

    def run_tests(self, rulename):
        for _, values in self.filenames.items():
            filename = values.get("filename")
            template = cfnlint.decode.cfn_yaml.load(filename)
            rules = Rules()
            rules.update(rules.create_from_custom_rules_file(rulename))
            runner = TemplateRunner(filename, template, ConfigMixIn({}), rules)
            return list(runner.run())



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from dataclasses import dataclass


@dataclass
class Mark:
    """Mark of line and column"""

    line: int = 1
    column: int = 1



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.codepipeline.CodepipelineStageActions import (
    CodepipelineStageActions,  # pylint: disable=E0401
)


class TestCodePipelineStageActions(BaseRuleTestCase):
    """Test CodePipeline Stage Actions Configuration"""

    def setUp(self):
        """Setup"""
        super(TestCodePipelineStageActions, self).setUp()
        self.collection.register(CodepipelineStageActions())
        self.success_templates = [
            "test/fixtures/templates/good/resources/codepipeline/stage_actions.yaml"
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_artifact_counts(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/action_artifact_counts.yaml",
            5,
        )

    def test_file_invalid_version(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/action_invalid_version.yaml",
            3,
        )

    def test_file_non_unique(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/action_non_unique.yaml",
            1,
        )

    def test_output_artifact_names_unique(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/output_artifact_non_unique.yaml",
            1,
        )

    def test_input_artifact_names_exist(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/input_artifact_not_exists.yaml",
            1,
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.codepipeline.CodepipelineStages import (
    CodepipelineStages,  # pylint: disable=E0401
)


class TestCodePipelineStages(BaseRuleTestCase):
    """Test CodePipeline Stages Configuration"""

    def setUp(self):
        """Setup"""
        super(TestCodePipelineStages, self).setUp()
        self.collection.register(CodepipelineStages())

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative_onestage(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/stages_one_stage.yaml",
            2,
        )

    def test_file_negative_no_source(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/stages_no_source.yaml",
            1,
        )

    def test_file_negative_second_stage(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/stages_second_stage.yaml",
            1,
        )

    def test_file_negative_non_unique(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/stages_non_unique.yaml",
            1,
        )

    def test_file_negative_only_source_types(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/codepipeline/stages_only_source.yaml",
            2,
        )

    def test_scenario_format(self):
        """Test scenario formatting"""
        rule = CodepipelineStages()

        self.assertEqual(
            rule._format_error_message("Test.", {"Condition": True}),
            'Test. When condition "Condition" is True',
        )
        self.assertEqual(rule._format_error_message("Test.", None), "Test.")



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from collections import deque

import pytest

from cfnlint.context import Path
from cfnlint.jsonschema import ValidationError

# ruff: noqa: E501
from cfnlint.rules.resources.RetentionPeriodOnResourceTypesWithAutoExpiringContent import (
    RetentionPeriodOnResourceTypesWithAutoExpiringContent,
)


@pytest.fixture(scope="module")
def rule():
    rule = RetentionPeriodOnResourceTypesWithAutoExpiringContent()
    yield rule


@pytest.fixture
def template():
    return {
        "Resources": {
            "MySqs": {
                "Type": "AWS::SQS::Queue",
            }
        }
    }


@pytest.fixture
def path():
    return Path(
        path=deque(["Resources", "MySqs", "Properties"]),
        cfn_path=deque(["Resources", "AWS::SQS::Queue", "Properties"]),
    )


@pytest.mark.parametrize(
    "name,instance,expected",
    [
        (
            "Valid version",
            {"MessageRetentionPeriod": "90"},
            [],
        ),
        (
            "Invalid type",
            [],
            [],
        ),
        (
            "Invalid when not specified",
            {},
            [
                ValidationError(
                    (
                        "'MessageRetentionPeriod' is a required property (The "
                        "default retention period will delete the data after "
                        "a pre-defined time. Set an explicit values to avoid "
                        "data loss on resource)"
                    ),
                    rule=RetentionPeriodOnResourceTypesWithAutoExpiringContent(),
                    schema_path=deque(["required"]),
                    validator="required",
                    validator_value=["MessageRetentionPeriod"],
                    instance={},
                )
            ],
        ),
    ],
)
def test_validate(name, instance, expected, rule, validator):
    errors = list(rule.validate(validator, False, instance, {}))
    # we use error counts in this one as the instance types are
    # always changing so we aren't going to hold ourselves up by that
    assert errors == expected, f"Test {name!r} got {errors!r}"


def test_validate_with_no_path(rule, validator):
    validator = validator.evolve(
        context=validator.context.path.evolve(
            path=Path(path=deque([]), cfn_path=deque([]))
        )
    )

    errors = list(rule.validate(validator, False, {}, {}))
    assert errors == []



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules import Rules
from cfnlint.rules.jsonschema.JsonSchema import JsonSchema
from cfnlint.runner import Runner


class TestBaseTemplate(BaseRuleTestCase):
    """Test base template"""

    def setUp(self):
        """Setup"""
        self.collection = Rules()
        self.collection.register(JsonSchema())

    def test_file_negative(self):
        """Failure test"""
        failure = "test/fixtures/templates/bad/template.yaml"
        try:
            Runner(self.collection, failure, True)
            self.assertEqual(1, 0)
        except Exception:
            pass



from cfnlint.jsonschema._typing import ResolutionResult, V, ValidationResult
from cfnlint.jsonschema.exceptions import ValidationError
from cfnlint.jsonschema.protocols import Validator
from cfnlint.jsonschema.validators import CfnTemplateValidator, StandardValidator

__all__ = [
    "ValidationResult",
    "ValidationError",
    "Validator",
    "CfnTemplateValidator",
    "StandardValidator",
    "ResolutionResult",
    "V",
]
